# Generated by Django 4.1.4 on 2022-12-10 07:38

import cast.blocks
from django.db import migrations
import wagtail.blocks
import wagtail.embeds.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0023_alter_spamfilter_model"),
    ]

    operations = [
        migrations.AlterField(
            model_name="homepage",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                    ("paragraph", wagtail.blocks.RichTextBlock()),
                    ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                    ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                ],
                use_json_field=True,
            ),
        ),
        migrations.AlterField(
            model_name="post",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    (
                        "overview",
                        wagtail.blocks.StreamBlock(
                            [
                                ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                                ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(icon="media", template="cast/video/video.html"),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(icon="media", template="cast/audio/audio.html"),
                                ),
                            ]
                        ),
                    ),
                    (
                        "detail",
                        wagtail.blocks.StreamBlock(
                            [
                                ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                                ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(icon="media", template="cast/video/video.html"),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(icon="media", template="cast/audio/audio.html"),
                                ),
                            ]
                        ),
                    ),
                ],
                use_json_field=True,
            ),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-19 14:37

import uuid

import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks
from django.conf import settings
from django.db import migrations, models

import cast.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0052_pagelogentry"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Audio",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("duration", models.DurationField(blank=True, null=True)),
                ("title", models.CharField(blank=True, max_length=255, null=True)),
                ("subtitle", models.CharField(blank=True, max_length=512, null=True)),
                ("m4a", models.FileField(blank=True, null=True, upload_to="cast_audio/")),
                ("mp3", models.FileField(blank=True, null=True, upload_to="cast_audio/")),
                ("oga", models.FileField(blank=True, null=True, upload_to="cast_audio/")),
                ("opus", models.FileField(blank=True, null=True, upload_to="cast_audio/")),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Blog",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("author", models.CharField(blank=True, default=None, max_length=255, null=True)),
                ("description", models.CharField(max_length=500)),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False)),
                ("email", models.EmailField(blank=True, default=None, max_length=254, null=True)),
                (
                    "comments_enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Whether comments are enabled for this blog.",
                        verbose_name="comments_enabled",
                    ),
                ),
                (
                    "itunes_categories",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="A json dict of itunes categories pointing to lists of subcategories. Taken from this list https://validator.w3.org/feed/docs/error/InvalidItunesCategory.html",
                        max_length=512,
                        verbose_name="itunes_categories",
                    ),
                ),
                (
                    "keywords",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="A comma-delimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                        max_length=255,
                        verbose_name="keywords",
                    ),
                ),
                (
                    "explicit",
                    models.PositiveSmallIntegerField(
                        choices=[(1, "yes"), (2, "no"), (3, "clean")],
                        default=1,
                        help_text="``Clean`` will put the clean iTunes graphic by it.",
                        verbose_name="explicit",
                    ),
                ),
                ("intro", wagtail.fields.RichTextField(blank=True)),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page", models.Model),
        ),
        migrations.CreateModel(
            name="Gallery",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="HomePage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "body",
                    wagtail.fields.StreamField(
                        [
                            ("heading", wagtail.blocks.CharBlock(classname="full title")),
                            ("paragraph", wagtail.blocks.RichTextBlock()),
                            ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                            ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                        ]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="Image",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "original",
                    models.ImageField(
                        height_field="original_height", upload_to="cast_images/originals", width_field="original_width"
                    ),
                ),
                ("original_height", models.PositiveIntegerField(blank=True, null=True)),
                ("original_width", models.PositiveIntegerField(blank=True, null=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cast_images",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ItunesArtWork",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "original",
                    models.ImageField(
                        height_field="original_height",
                        upload_to="cast_images/itunes_artwork",
                        width_field="original_width",
                    ),
                ),
                ("original_height", models.PositiveIntegerField(blank=True, null=True)),
                ("original_width", models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Request",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("ip", models.GenericIPAddressField()),
                ("timestamp", models.DateTimeField()),
                ("status", models.PositiveSmallIntegerField()),
                ("size", models.PositiveIntegerField()),
                ("referer", models.CharField(blank=True, max_length=2048, null=True)),
                ("user_agent", models.CharField(blank=True, max_length=1024, null=True)),
                (
                    "method",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "GET"),
                            (2, "HEAD"),
                            (3, "POST"),
                            (4, "PUT"),
                            (5, "PATCH"),
                            (6, "DELETE"),
                            (7, "OPTIONS"),
                            (8, "CONNECT"),
                            (8, "TRACE"),
                        ]
                    ),
                ),
                ("path", models.CharField(max_length=1024)),
                (
                    "protocol",
                    models.PositiveSmallIntegerField(choices=[(1, "HTTP/1.0"), (2, "HTTP/1.1"), (3, "HTTP/2.0")]),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Video",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("original", models.FileField(upload_to="cast_videos/")),
                ("poster", models.ImageField(blank=True, null=True, upload_to="cast_videos/poster/")),
                ("poster_seconds", models.FloatField(default=1)),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Post",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False)),
                ("pub_date", models.DateTimeField(blank=True, null=True)),
                ("visible_date", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "keywords",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                        max_length=255,
                        verbose_name="keywords",
                    ),
                ),
                (
                    "explicit",
                    models.PositiveSmallIntegerField(
                        choices=[(1, "yes"), (2, "no"), (3, "clean")],
                        default=1,
                        help_text="``Clean`` will put the clean iTunes graphic by it.",
                        verbose_name="explicit",
                    ),
                ),
                (
                    "block",
                    models.BooleanField(
                        default=False,
                        help_text="Check to block this episode from iTunes because <br />its content might cause the entire show to be <br />removed from iTunes.",
                        verbose_name="block",
                    ),
                ),
                (
                    "comments_enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Whether comments are enabled for this post.",
                        verbose_name="comments_enabled",
                    ),
                ),
                (
                    "body",
                    wagtail.fields.StreamField(
                        [
                            ("heading", wagtail.blocks.CharBlock(classname="full title")),
                            ("paragraph", wagtail.blocks.RichTextBlock()),
                            ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                            ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                        ]
                    ),
                ),
                ("audios", models.ManyToManyField(blank=True, to="cast.Audio")),
                ("galleries", models.ManyToManyField(blank=True, to="cast.Gallery")),
                ("images", models.ManyToManyField(blank=True, to="cast.Image")),
                (
                    "parent_blog",
                    models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to="cast.blog"),
                ),
                (
                    "podcast_audio",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="posts",
                        to="cast.audio",
                    ),
                ),
                ("videos", models.ManyToManyField(blank=True, to="cast.Video")),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page", models.Model),
        ),
        migrations.AddField(
            model_name="gallery",
            name="images",
            field=models.ManyToManyField(to="cast.Image"),
        ),
        migrations.AddField(
            model_name="gallery",
            name="user",
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name="File",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("original", models.FileField(upload_to="cast_files/")),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="blog",
            name="itunes_artwork",
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to="cast.itunesartwork"
            ),
        ),
        migrations.CreateModel(
            name="ChapterMark",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("start", models.TimeField(verbose_name="Start time of chaptermark")),
                ("title", models.CharField(max_length=255)),
                ("link", models.URLField(blank=True, max_length=2000, null=True)),
                ("image", models.URLField(blank=True, max_length=2000, null=True)),
                (
                    "audio",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, related_name="chaptermarks", to="cast.audio"
                    ),
                ),
            ],
            options={
                "unique_together": {("audio", "start")},
            },
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-21 08:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0017_alter_post_body"),
    ]

    operations = [
        migrations.AlterField(
            model_name="chaptermark",
            name="start",
            field=models.TimeField(unique=True, verbose_name="Start time of chaptermark"),
        ),
    ]



import pytest
from django.urls import reverse


class TestVideoUpload:
    @pytest.mark.django_db
    def test_upload_video_not_authenticated(self, client, minimal_mp4):
        upload_url = reverse("cast:api:upload_video")

        minimal_mp4.seek(0)
        r = client.post(upload_url, {"original": minimal_mp4})
        # redirect to login
        assert r.status_code == 302

    @pytest.mark.django_db
    def test_upload_video_authenticated(self, client, user, minimal_mp4):
        # login
        r = client.login(username=user.username, password=user._password)

        self.called_create_poster = False

        def set_called_create_poster():
            self.called_create_poster = True

        # mock create poster?
        # Video._saved_create_poster = Video._create_poster
        # Video._create_poster = lambda x: set_called_create_poster()

        # upload
        upload_url = reverse("cast:api:upload_video")
        minimal_mp4.seek(0)
        r = client.post(upload_url, {"original": minimal_mp4})

        # unmock
        # Video._create_poster = Video._saved_create_poster

        assert r.status_code == 201
        assert int(r.content.decode("utf-8")) > 0

        # check mocked function has been called - no longer necessary since we use
        # a real mp4 now.
        # assert self.called_create_poster



# import pytest
#
# from cast import upload_handler
# from cast.models import Audio, Image, Video
#
#
# class TestUploadHandler:
#     def test_upload_handler_unknown(self, request_factory):
#         request = request_factory.post("")
#         request.FILES["original"] = "foobar.xls"
#         with pytest.raises(KeyError):
#             form_class, context = upload_handler(request)
#
#     def test_upload_handler_image(self, request_factory):
#         request = request_factory.post("")
#         for ending in ("jpg", "jpeg", "png", "gif"):
#             request.FILES["original"] = f"foobar.{ending}"
#             form_class, context = upload_handler(request)
#             assert form_class._meta.model == Image
#
#     def test_upload_handler_video(self, request_factory):
#         request = request_factory.post("")
#         for ending in ("mp4", "mov", "m4v"):
#             request.FILES["original"] = f"foobar.{ending}"
#             form_class, context = upload_handler(request)
#             assert form_class._meta.model == Video
#
#     @pytest.mark.django_db
#     def test_upload_handler_audio(self, request_factory):
#         request = request_factory.post("")
#         for ending in ("mp3", "ogg", "m4a", "opus"):
#             request.FILES["original"] = f"foobar.{ending}"
#             form_class, context = upload_handler(request)
#             assert form_class._meta.model == Audio
#
#     @pytest.mark.django_db
#     def test_upload_handler_audio_ogg(self, request_factory):
#         """For ogg the field name is different from the file ending."""
#         request = request_factory.post("")
#         request.FILES["original"] = "foobar.ogg"
#         form_class, context = upload_handler(request)
#         assert context["upload_field_name"] == "oga"



import pytest

from cast.admin import (
    AdminUserMixin,
    SpamfilterModelAdmin,
    VideoModelAdmin,
    cache_file_sizes,
    retrain,
)
from cast.models import SpamFilter, Video


def test_spamfilter_model_admin():
    expected_performance = {
        "spam": {"precision": 0.5, "recall": 0.5},
        "ham": {"precision": 0.5, "recall": 0.5},
    }
    spamfilter = SpamFilter(performance=expected_performance)
    sma = SpamfilterModelAdmin(SpamFilter, None)
    assert sma.spam(spamfilter) == expected_performance["spam"]
    assert sma.ham(spamfilter) == expected_performance["ham"]


class SpySpamfilter:
    got_training_data = False
    retrained = False

    def get_training_data_comments(self):
        self.got_training_data = True
        return []

    def retrain_from_scratch(self, train):
        self.retrained = True


def test_retrain():
    spy = SpySpamfilter()
    retrain(None, None, [spy])
    assert spy.got_training_data
    assert spy.retrained


@pytest.mark.django_db
def test_video_model_admin_calc_poster(mocker):
    class MockedForm:
        cleaned_data = {"poster": False}

    mocked_super = mocker.patch("cast.admin.ModelAdmin.save_model")
    vma = VideoModelAdmin(Video, None)

    # change=True, poster=False -> calc_poster=False
    vma.save_model(None, Video(), MockedForm(), True)
    processed_video = mocked_super.call_args[0][1]
    assert not processed_video.calc_poster

    # change=False, poster=False -> calc_poster=True
    vma.save_model(None, Video(), MockedForm(), False)
    processed_video = mocked_super.call_args[0][1]
    assert processed_video.calc_poster


def test_cache_file_sizes():
    class SpyAudio:
        cached = False

        def size_to_metadata(self):
            self.cached = True

        def save(self):
            pass

    spy = SpyAudio()
    cache_file_sizes(None, None, [spy])
    assert spy.cached


def test_admin_user_mixin():
    class SpyRequest:
        user = "foobar"

    aum = AdminUserMixin()
    initial_data = aum.get_changeform_initial_data(SpyRequest())
    assert initial_data == {"user": SpyRequest.user, "author": SpyRequest.user}



from django.db import models
from wagtail.snippets.models import register_snippet


@register_snippet
class PostCategory(models.Model):
    """Post category snippet for grouping posts."""

    name = models.CharField(max_length=255, unique=True, help_text="The name for this category")
    slug = models.SlugField(verbose_name="slug", unique=True, help_text="A slug to identify posts by this category")

    class Meta:
        verbose_name = "Post Category"
        verbose_name_plural = "Post Categories"
        ordering = ["name"]

    def __str__(self) -> str:
        return self.name



import pytest
from django.urls import reverse


class TestPublished:
    pytestmark = pytest.mark.django_db

    def test_get_only_published_entries(self, client, unpublished_post):
        bp = unpublished_post
        feed_url = reverse("cast:latest_entries_feed", kwargs={"slug": bp.blog.slug})

        r = client.get(feed_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "xml" in content
        assert bp.title not in content

    def test_get_post_detail_not_published_not_auth(self, client, unpublished_post):
        post = unpublished_post
        detail_url = post.get_url()

        r = client.get(detail_url)
        assert r.status_code == 404

        content = r.content.decode("utf-8")
        assert post.title not in content



import pytest

from cast.models import Post


@pytest.mark.django_db
def test_post_image_in_get_all_images_from_queryset(post_with_image):
    # Given a post with an image
    [post_image] = post_with_image.images.all()
    # When we get all images from the queryset including the post
    post_queryset = Post.objects.filter(pk=post_with_image.pk).prefetch_related("images")
    all_images = {image for image_type, image in Post.get_all_images_from_queryset(post_queryset)}
    # Then the image should be in the list
    assert post_image in all_images


@pytest.mark.django_db
def test_post_image_from_gallery_in_get_all_images_from_queryset(post_with_gallery):
    # Given a post with a gallery with an image
    [post_gallery] = post_with_gallery.galleries.all()
    [gallery_image] = post_gallery.images.all()
    # When we get all images from the queryset including the post
    post_queryset = Post.objects.filter(pk=post_with_gallery.pk).prefetch_related("galleries__images")
    all_images = {image for image_type, image in Post.get_all_images_from_queryset(post_queryset)}
    # Then the image should be in the list
    assert gallery_image in all_images


@pytest.mark.django_db
def test_post_get_all_renditions_from_queryset(post_with_image):
    post_queryset = Post.objects.filter(pk=post_with_image.pk).prefetch_related("galleries__images")
    all_renditions = list(Post.get_all_renditions_from_queryset(post_queryset))
    assert all_renditions == []


class StubWagtailImage:
    class File:
        name = "test.jpg"

    pk = 1
    width = 6000
    height = 4000
    file = File()


def test_post_get_all_required_filter_strings():
    # Given an iterator ofr images with types
    images = [("regular", StubWagtailImage())]
    # When we get all required filter strings for the image
    required_filter_strings = {fs for ipk, fs in Post.get_all_filterstrings(images)}  # type: ignore
    # Then the required filter strings should be in the list
    assert "width-1110" in required_filter_strings


class RenditionQuerysetStub:
    pk = 3

    def values_list(self, *_args):
        return [
            (self.pk, 2, "width-1110"),  # one obsolete rendition, since image_id != 1
            (self.pk, 2, "max-165x165"),  # one wagtailadmin thumbnail that is not obsolete
        ]


@pytest.mark.django_db
def test_post_get_obsolete_and_missing_rendition_strings(mocker):
    # Given an iterator of images with types
    images = [("regular", StubWagtailImage())]
    # And a stub for the Rendition queryset
    rendition_queryset = RenditionQuerysetStub()
    mocker.patch("cast.models.pages.Rendition.objects.filter", return_value=rendition_queryset)
    # When we get all obsolete and missing renditions for the images
    obsolete_renditions, missing_renditions = Post.get_obsolete_and_missing_rendition_strings(images)
    # Then the obsolete renditions should be empty
    assert obsolete_renditions == {rendition_queryset.pk}
    # Then the missing renditions should be in the list
    assert "width-1110" in missing_renditions[1]



# Generated by Django 4.1.4 on 2023-02-02 21:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0029_add_metadata_field_to_audio"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="post",
            name="pub_date",
        ),
    ]



import os
from collections.abc import Iterable

from django.core.files.storage import Storage


def storage_walk_paths(storage: Storage, cur_dir: str = "") -> Iterable[str]:
    dirs, files = storage.listdir(cur_dir)
    for directory in dirs:
        new_dir = os.path.join(cur_dir, directory)
        for path in storage_walk_paths(storage, cur_dir=new_dir):
            yield path
    for file_name in files:
        path = os.path.join(cur_dir, file_name)
        yield path



from django.core.management.base import BaseCommand

from cast.utils import storage_walk_paths

from .storage_backend import get_production_and_backup_storage


class Command(BaseCommand):
    help = (
        "show size of media files on production storage backend"
        "(requires Django >= 4.2 and production and backup storage configured)"
    )

    @staticmethod
    def show_usage(paths):
        video_endings = {"mov", "mp4"}
        image_endings = {"jpg", "jpeg", "png"}
        image, video, misc = 0, 0, 0
        for path, size in paths.items():
            ending = path.split(".")[-1].lower()
            if ending in video_endings:
                video += size
            elif ending in image_endings:
                image += size
            else:
                misc += size
        unit = 2**20  # MB
        print(f"video usage: {video / unit}")
        print(f"image usage: {image / unit}")
        print(f"misc  usage: {misc / unit}")
        print(f"total usage: {sum(paths.values()) / unit}")

    @staticmethod
    def get_paths_with_sizes_for(storage_backend):
        paths = {}
        for path in storage_walk_paths(storage_backend):
            size = storage_backend.size(path)
            paths[path] = size
            print(path, size / 2**20)
        return paths

    def handle(self, *args, **options):
        production, _ = get_production_and_backup_storage()
        paths = self.get_paths_with_sizes_for(production)
        self.show_usage(paths)



from datetime import timedelta

import pytest

from cast.models.audio import Audio, ChapterMark, sync_chapter_marks


class TestAudioModel:
    pytestmark = pytest.mark.django_db

    def test_get_file_formats(self, audio):
        assert audio.file_formats == "m4a"

    def test_get_file_names(self, audio):
        assert "test" in audio.get_audio_file_names()

    def test_get_name(self, audio):
        audio.title = None  # make sure name is provided by file
        assert audio.name == "test"

    def test_get_name_with_title(self, audio):
        title = "foobar"
        audio.title = title
        assert audio.name == title

    def test_audio_str(self, audio):
        audio.title = None  # make sure name is provided by file
        assert "1 - test" == str(audio)

    def test_audio_get_all_paths(self, audio):
        assert "cast_audio/test.m4a" in audio.get_all_paths()

    def test_audio_duration(self, audio):
        duration = audio._get_audio_duration(audio.m4a.path)
        assert duration in (timedelta(microseconds=746667), timedelta(microseconds=700000))

    def test_audio_duration_no_match(self, mocker):
        mocker.patch("cast.models.audio.subprocess.check_output", return_value=b"foobar")
        with pytest.raises(ValueError):
            Audio._get_audio_duration("https://example.com/test.m4a")

    def test_audio_create_duration(self, audio):
        duration = "00:01:01.00"
        audio._get_audio_duration = lambda x: duration
        audio.create_duration()
        assert audio.duration == duration

    def test_audio_create_duration_no_duration(self, audio, mocker):
        class Field:
            url = "https://example.com/to.mp3"
            path = "/tmp/to.mp3"

        field = Field()
        get_audio_duration = mocker.patch("cast.models.audio.Audio._get_audio_duration", return_value="duration")
        mocker.patch("cast.models.audio.Audio.uploaded_audio_files", [("mp3", field)])
        audio.create_duration()
        get_audio_duration.assert_called_once_with(field.url)

    def test_audio_create_duration_no_file_field(self, audio, mocker):
        class Field:  # not a FileField
            foo = "bar"

        field = Field()
        get_audio_duration = mocker.patch("cast.models.audio.Audio._get_audio_duration", return_value="duration")
        mocker.patch("cast.models.audio.Audio.uploaded_audio_files", [("mp3", field)])
        audio.create_duration()
        assert get_audio_duration.call_count == 0

    def test_audio_audio_without_file_fields(self, audio, mocker):
        class Field:  # not a FileField
            foo = "bar"

        field = Field()
        mocker.patch("cast.models.audio.Audio.uploaded_audio_files", [("mp3", field)])
        assert audio.audio == []

    def test_audio_get_chaptermark_data_from_file_https_not_in_url(self, audio, mocker):
        class RunReturn:
            stdout = '{"foo": "bar"}'

        class Field:
            url = "https://example.com/to.mp3"
            path = "/tmp/to.mp3"

        field = Field()
        mocker.patch("cast.models.audio.getattr", return_value=field)
        run = mocker.patch("cast.models.audio.subprocess.run", return_value=RunReturn())
        mocker.patch("cast.models.audio.Audio.clean_ffprobe_chaptermarks")
        _ = audio.get_chaptermark_data_from_file("mp3")
        assert field.url == run.call_args[0][0][2]

    def test_audio_podlove_url(self, audio):
        assert audio.podlove_url == "/cast/api/audios/podlove/1"

    def test_get_episode_url_from_audio(self, episode):
        audio = episode.podcast_audio

        # happy path - audio has episode and episode_id is set
        audio.set_episode_id(episode.pk)
        assert "http" in audio.episode_url

        # happy path - audio is only used by one episode
        del audio._episode_id
        assert "http" in audio.episode_url

        # sad path - audio is not used by any episode
        episode.unpublish()
        episode.podcast_audio = None
        episode.save()
        assert audio.episode_url is None

    def test_get_episode_url_from_audio_with_multiple_episodes(self, episode, podcast_episode_with_same_audio):
        audio = episode.podcast_audio

        # sad path - audio is used by multiple episodes
        assert audio.episode_url is None

        # happy path - audio is used by multiple episodes but episode_id is set
        audio.set_episode_id(episode.pk)
        assert audio.episode_url == episode.full_url

        audio.set_episode_id(podcast_episode_with_same_audio.pk)
        assert audio.episode_url == podcast_episode_with_same_audio.full_url

    def test_audio_get_chaptermark_data_from_file_empty_on_value_error(self, audio):
        assert audio.get_chaptermark_data_from_file("mp3") == []

    def test_audio_test_clean_ffprobe_chaptermarks(self):
        ffprobe_chaptermarks = {
            "chapters": [
                {
                    "id": 0,
                    "time_base": "1/1000",
                    "start": 0,
                    "start_time": "0.000000",
                    "end": 155343,
                    "end_time": "155.343000",
                    # chapter marks with empty title should be filtered
                    "tags": {"title": ""},
                },
                {
                    "id": 1,
                    "time_base": "1/1000",
                    "start": 155343,
                    "start_time": "155.343000",
                    "end": 617117,
                    "end_time": "617.117000",
                    "tags": {"title": "News aus der Szene"},
                },
                {
                    "id": 2,
                    "time_base": "1/1000",
                    "start": 617117,
                    "start_time": "617.117000",
                    "end": 1266062,
                    "end_time": "1266.062000",
                    "tags": {"title": "Django Async"},
                },
            ]
        }
        cleaned_chaptermarks = Audio.clean_ffprobe_chaptermarks(ffprobe_chaptermarks)
        assert len(cleaned_chaptermarks) == 2
        assert cleaned_chaptermarks == [
            {"start": "155.343000", "title": "News aus der Szene"},
            {"start": "617.117000", "title": "Django Async"},
        ]

    def test_chapters_as_text_chaptermarks_not_empty(self, audio, mocker):
        class Mark:
            original_line = "original line"

        mark = Mark()

        class ChapterMarks:
            @staticmethod
            def order_by(_order):
                return [mark]

        mocker.patch("cast.models.audio.Audio.chaptermarks", ChapterMarks())
        assert mark.original_line in audio.chapters_as_text

    def test_write_audio_file_size_to_cache(self, audio):
        expected_size = audio.m4a.size
        audio.size_to_metadata()
        assert audio.data["size"]["m4a"] == expected_size

    def test_read_audio_file_size_from_cache(self, audio):
        # when size is not in cache, it should be read from file
        expected_file_size = audio.m4a.size
        audio.data = {}
        assert audio.get_file_size("m4a") == expected_file_size

        # when size is in cache, it should be read from cache
        expected_cache_size = 123
        audio.data = {"size": {"m4a": expected_cache_size}}
        assert audio.get_file_size("m4a") == expected_cache_size

        # when file field is null, return 0
        audio.mp3 = None
        assert audio.get_file_size("mp3") == 0

    def test_save_without_cache_file_sizes(self, audio, mocker):
        mocker.patch("cast.models.audio.TimeStampedModel.save")
        audio.data = expected_data = {}
        audio.save(generate_duration=False, cache_file_sizes=False)
        assert audio.data == expected_data


class TestChapterMarkModel:
    pytestmark = pytest.mark.django_db

    def test_chaptermark_original_line(self, chaptermarks):
        chaptermark = chaptermarks[0]
        assert chaptermark.original_line == "00:01:01.234 introduction  "

    def test_chaptermark_original_line_link(self, chaptermarks):
        link = "https://foobar.com"
        chaptermark = chaptermarks[0]
        chaptermark.link = link
        assert chaptermark.original_line == f"00:01:01.234 introduction {link} "

    def test_chaptermark_original_line_image(self, chaptermarks):
        image = "https://foobar.com/blub.jpg"
        chaptermark = chaptermarks[0]
        chaptermark.image = image
        assert chaptermark.original_line == f"00:01:01.234 introduction  {image}"


def start_strings_to_chaptermarks(audio, start_strings):
    chaptermarks = []
    for start_string in start_strings:
        cm = ChapterMark(audio=audio, start=start_string, title="foobar")
        cm.full_clean()  # convert string to datetime.time
        chaptermarks.append(cm)
    return chaptermarks


def chaptermarks_are_equal(actual, expected):
    if len(actual) != len(expected):
        return False
    attributes_to_compare = ["start", "title"]
    for a, b in zip(actual, expected):
        for attr in attributes_to_compare:
            if getattr(a, attr) != getattr(b, attr):
                return False
    return True


@pytest.mark.parametrize(
    "from_database, from_cms, expected_to_add, expected_to_update, expected_to_remove",
    [
        # from_database, from_cms, expected_to_add, expected_to_update, expected_to_remove
        ([], [], [], [], []),
        ([], ["0:0"], ["0:0"], [], []),  # add chaptermark
        (["0:0"], ["0:0"], [], [], []),  # update chaptermark -> empty because equal
        (["0:0"], [], [], [], ["0:0"]),  # remove chaptermark
        # add, update and remove together
        (["0:0", "0:2"], ["0:0", "0:1"], ["0:1"], [], ["0:2"]),
    ],
)
@pytest.mark.django_db
def test_sync_chapter_marks(from_database, from_cms, expected_to_add, expected_to_update, expected_to_remove, audio):
    args = []
    for start_string_list in [from_database, from_cms, expected_to_add, expected_to_update, expected_to_remove]:
        args.append(start_strings_to_chaptermarks(audio, start_string_list))
    from_database, from_cms, expected_to_add, expected_to_update, expected_to_remove = args
    actual_to_add, actual_to_update, actual_to_remove = sync_chapter_marks(from_database, from_cms)
    for actual, expected in zip(
        [actual_to_add, actual_to_update, actual_to_remove], [expected_to_add, expected_to_update, expected_to_remove]
    ):
        assert chaptermarks_are_equal(actual, expected)


@pytest.mark.django_db
def test_sync_chapter_marks_update_changed_chaptermarks(audio):
    cm1 = ChapterMark(audio=audio, start="0:0", title="foobar")
    cm1.full_clean()
    cm2 = ChapterMark(audio=audio, start="0:0", title="changed")
    cm2.full_clean()
    actual_to_add, actual_to_update, actual_to_remove = sync_chapter_marks([cm1], [cm2])
    assert len(actual_to_update) == 1



from django.db import models

from model_utils.models import TimeStampedModel


class ItunesArtWork(TimeStampedModel):
    original = models.ImageField(
        upload_to="cast_images/itunes_artwork",
        height_field="original_height",
        width_field="original_width",
    )
    original_height = models.PositiveIntegerField(blank=True, null=True)
    original_width = models.PositiveIntegerField(blank=True, null=True)



import string
from collections.abc import Iterable, Mapping
from datetime import datetime
from typing import Any, Optional, cast

import django_filters
from django.core import validators
from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import UploadedFile
from django.db import models
from django.db.models.fields import BLANK_CHOICE_DASH
from django.db.models.functions import TruncMonth
from django.forms import Widget
from django.forms.renderers import BaseRenderer
from django.forms.utils import flatatt
from django.http import QueryDict
from django.utils.datastructures import MultiValueDict
from django.utils.encoding import force_str
from django.utils.safestring import SafeText, mark_safe
from django.utils.translation import gettext as _
from django_filters.fields import ChoiceField as FilterChoiceField
from taggit.models import Tag
from wagtail.models import PageQuerySet

from cast import appsettings
from cast.models.snippets import PostCategory


class CountFacetWidget(Widget):
    data: QueryDict

    def __init__(self, attrs: Optional[dict[str, str]] = None):
        super().__init__(attrs)
        self.choices: list[tuple[str, str]] = []

    def value_from_datadict(  # type: ignore[override]
        self, data: Mapping[str, Iterable[Any]], files: MultiValueDict[str, UploadedFile], name: str
    ) -> str:
        data_for_super = cast(dict[str, Any], data)  # make mypy happy
        value = super().value_from_datadict(data_for_super, files, name)
        self.data: QueryDict = cast(QueryDict, data)
        return value

    def render(
        self, name: str, value: Any, attrs: Optional[dict[str, Any]] = None, renderer: Optional[BaseRenderer] = None
    ) -> SafeText:
        if value is None:
            value = ""
        final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
        output = ["<div%s>" % flatatt(final_attrs)]
        options = self.render_options([value], name)
        if options:
            output.append(options)
        output.append("</div>")
        return mark_safe("\n".join(output))

    def render_options(self, selected_choices: list[str], name: str) -> str:
        selected_choices_set = {force_str(v) for v in selected_choices}
        output = []
        for option_value, option_label in self.choices:
            if isinstance(option_label, (list, tuple)):
                for option in option_label:
                    output.append(self.render_option(name, selected_choices_set, option_value, option))
            else:
                output.append(self.render_option(name, selected_choices_set, option_value, option_label))
        return "\n".join(output)

    def render_option(self, name: str, selected_choices: set[str], option_value: str, option_label: str) -> str:
        option_value = force_str(option_value)
        if option_label == BLANK_CHOICE_DASH[0][1]:
            option_label = _("All")

        # remove page from querystring, because otherwise the pagination breaks
        # filters like date facets, str to make mypy happy
        data_dict = {k: str(v) for k, v in self.data.items() if k != "page"}
        data = QueryDict("", mutable=True)
        data.update(data_dict)
        data[name] = option_value

        # build the option string
        url = data.urlencode()
        option_attrs, hidden_input = "", ""
        if option_value in selected_choices:
            # the current option is already selected, so add a hidden input field
            # to preserve the selection and add the class "selected" to the link
            option_attrs = ' class="selected"'
            hidden_input = f'<input type="hidden" name="{name}" value="{option_value}">'
        option_string = self.option_string() % {
            "attrs": option_attrs,
            "query_string": url,
            "label": force_str(option_label),
            "hidden_input": hidden_input,
        }
        return option_string

    @staticmethod
    def option_string() -> str:
        return (
            '<div class="cast-date-facet-item"><a%(attrs)s href="?%(query_string)s">%(label)s</a>'
            "%(hidden_input)s</div>"
        )


def parse_date_facets(value: str) -> datetime:
    """Split into function, because it needs to be imported by the post list view."""
    # clean up value a bit, because otherwise sql-injection
    # search requests are spamming the logfile with garbage -> analytics won't work
    allowed = set(string.digits + "-")
    value = "".join([c for c in value if c in allowed])
    year_month = datetime.strptime(value, "%Y-%m")
    return year_month


DateFacetCounts = dict[datetime, int]
Choices = list[tuple[str, str]]


class DateFacetChoicesMixin:
    """Just a way to pass the facet counts to the field which displays the choice."""

    facet_counts: DateFacetCounts = {}
    extra: dict[str, Any] = {}

    @staticmethod
    def transform_facet_counts_to_choices(facet_counts: DateFacetCounts) -> Choices:
        """
        Transform the facet counts into a list of choices for the field.
        """
        facet_count_choices = []
        for year_month, count in sorted(facet_counts.items()):
            date_str = year_month.strftime("%Y-%m")
            label = f"{date_str} ({count})"
            facet_count_choices.append((date_str, label))
        return facet_count_choices

    def set_field_choices(self, choices: Choices) -> None:
        """
        Set the choices on the field. Also remove the cached _field
        attribute so that the field is recreated with the new choices
        next time it is accessed.

        See django_filters/filters.py:Filter::field for more info.
        """
        self.extra["choices"] = choices
        delattr(self, "_field")


class AllDateChoicesField(FilterChoiceField):
    def valid_value(self, value: str) -> bool:
        """
        Allow all values instead of explicit choices but still validate
        against parse_date_facets to make sure the facet is parseable
        into a date.
        """
        try:
            parse_date_facets(value)
            return True
        except ValueError:
            return False


class DateFacetFilter(DateFacetChoicesMixin, django_filters.filters.ChoiceFilter):
    field_class = AllDateChoicesField

    def filter(self, qs: models.QuerySet, value: str) -> models.QuerySet:
        if len(value) == 0:
            # don't filter if value is empty
            return qs
        year_month = parse_date_facets(value)
        year = year_month.year
        month = year_month.month
        filtered = qs.filter(visible_date__year=year, visible_date__month=month)
        return filtered

    def set_facet_counts(self, queryset: models.QuerySet) -> None:
        """
        Fetch facet counts for filtered queryset and set the field
        choices to the facet counts.
        """
        self.facet_counts = {
            month: num_posts
            for month, num_posts in (
                queryset.order_by()
                .annotate(month=TruncMonth("visible_date"))
                .values("month")
                .annotate(num_posts=models.Count("pk"))
            ).values_list("month", "num_posts")
        }
        choices = self.transform_facet_counts_to_choices(self.facet_counts)
        self.set_field_choices(choices)


class SlugChoicesField(FilterChoiceField):
    def valid_value(self, value: str) -> bool:
        """
        Used to determine if the value provided by the user can be used
        to filter the queryset. Return early if value is not a string,
        use the slug validator to check if the value is a valid slug.
        """
        if not isinstance(value, str):
            return False
        try:
            validators.validate_slug(value)
            return True
        except ValidationError:
            return False


FacetCounts = dict[str, tuple[str, int]]


class CountChoicesMixin:
    """Just a way to pass the facet counts to the field which displays the choice."""

    facet_counts: FacetCounts = {}
    has_facets_with_posts: bool = False
    extra: dict[str, Any] = {}

    @staticmethod
    def fetch_facet_counts(post_queryset: models.QuerySet, count_queryset: models.QuerySet) -> FacetCounts:
        """
        Fetch the facet counts for all facets with post count > 0.
        """
        return {
            slug: (name, num_posts)
            for slug, name, num_posts in count_queryset.annotate(
                num_posts=models.Count("post", filter=models.Q(post__in=post_queryset))
            )
            .filter(num_posts__gt=0)
            .values_list("slug", "name", "num_posts")
        }

    @staticmethod
    def transform_facet_counts_to_choices(facet_counts: FacetCounts) -> Choices:
        """
        Transform the facet counts into a list of choices for the field.
        """
        facet_count_choices = []
        for slug, (name, count) in sorted(facet_counts.items()):
            label = f"{name} ({count})"
            facet_count_choices.append((slug, label))
        return facet_count_choices

    def set_field_choices(self, choices: Choices) -> None:
        """
        Set the choices on the field. Also remove the cached _field
        attribute so that the field is recreated with the new choices
        next time it is accessed.

        Also set the has_facets_with_posts attribute to True if there
        are any facets with posts. This is used to determine if the
        form field should be hidden.
        """
        self.extra["choices"] = choices
        delattr(self, "_field")
        self.has_facets_with_posts = len(choices) > 0

    @property
    def hide_form_field(self) -> bool:
        """True if there are no facets containing posts."""
        return not self.has_facets_with_posts


class CategoryFacetFilter(CountChoicesMixin, django_filters.filters.ChoiceFilter):
    field_class = SlugChoicesField

    def filter(self, qs: models.QuerySet, value: str):
        # Check if value is provided (not None and not an empty list)
        if value:
            return qs.filter(categories__slug__in=[value])
        return qs

    def set_facet_counts(self, queryset: models.QuerySet) -> None:
        """
        Fetch the facet counts for all facets with post count > 0
        and set the choices on the field.
        """
        self.facet_counts = self.fetch_facet_counts(queryset, PostCategory.objects.all())
        choices = self.transform_facet_counts_to_choices(self.facet_counts)
        self.set_field_choices(choices)


class TagFacetFilter(CountChoicesMixin, django_filters.filters.ChoiceFilter):
    field_class = SlugChoicesField
    facet_count_key = "tags"

    def filter(self, qs: models.QuerySet, value: str):
        # Check if value is provided (not None and not an empty list)
        if value:
            return qs.filter(tags__slug__in=[value])
        return qs

    def set_facet_counts(self, queryset: models.QuerySet) -> None:
        """
        Fetch the facet counts for all facets with post count > 0
        and set the choices on the field.

        Cannot use PostTag.objects.annotate here because the group by clause
        would be wrong (GROUP BY "cast_posttag"."id" instead of "tag"."id")
        """
        self.facet_counts = self.fetch_facet_counts(queryset, Tag.objects.all())
        choices = self.transform_facet_counts_to_choices(self.facet_counts)
        self.set_field_choices(choices)


class PostFilterset(django_filters.FilterSet):
    """
    A filterset for posts. It is used to filter the posts faceted navigation
    style. The filterset facets are configured via the CAST_FILTERSET_FACETS setting.

    The main reason why this code is confusing is that the facets are used
    as the choices for the filterset fields. But the facets are only available
    if the queryset is filtered. So the choices of the fields need to be set
    after the queryset is filtered.
    """

    search = django_filters.CharFilter(field_name="search", method="fulltext_search", label="Search")
    date = django_filters.DateFromToRangeFilter(
        field_name="visible_date",
        label="Date",
        widget=django_filters.widgets.DateRangeWidget(  # type: ignore
            attrs={"type": "date", "placeholder": "YYYY/MM/DD"}
        ),  # type: ignore
    )
    # FIXME Maybe use ModelMultipleChoiceFilter for categories? Couldn't get it to work for now, though.
    #   - one problem was that after setting choices via the choices parameter, Django randomly
    #     complained about models not being available before app start etc.
    category_facets = CategoryFacetFilter(
        field_name="category_facets",
        label="Categories",
        # choices do not need to be set, since they are transported from facet counts
        # into the extra dict of the field via CountChoicesMixin
        widget=CountFacetWidget(attrs={"class": "cast-date-facet-container"}),
    )
    tag_facets = TagFacetFilter(
        field_name="tag_facets",
        label="Tags",
        # choices do not need to be set, since they are transported from facet counts
        # into the extra dict of the field via CountChoicesMixin
        widget=CountFacetWidget(attrs={"class": "cast-date-facet-container"}),
    )
    date_facets = DateFacetFilter(
        field_name="date_facets",
        label="Date Facets",
        widget=CountFacetWidget(attrs={"class": "cast-date-facet-container"}),
    )
    o = django_filters.OrderingFilter(
        fields=(("visible_date", "visible_date"),),
        field_labels={"visible_date": "Date"},
    )

    class Meta:
        fields = appsettings.CAST_FILTERSET_FACETS

    def __init__(
        self,
        data: Optional[QueryDict] = None,
        queryset: Optional[models.QuerySet] = None,
    ):
        if data is None:
            data = QueryDict("")
        if queryset is None:
            from cast.models.pages import Post

            queryset = Post.objects.none()
        super().__init__(data=data, queryset=queryset)
        # Remove filters which are not configured in the settings
        configured_filters = set(appsettings.CAST_FILTERSET_FACETS)
        for filter_name in self.filters.copy().keys():
            if filter_name not in configured_filters:
                del self.filters[filter_name]
        self.set_facet_counts(self.qs)
        self.remove_form_fields_that_should_be_hidden()

    def set_facet_counts(self, queryset: models.QuerySet) -> None:
        facet_queryset = queryset
        for filter_name, post_filter in self.filters.items():
            if hasattr(post_filter, "set_facet_counts"):
                post_filter.set_facet_counts(facet_queryset)
                self.form.fields[filter_name] = post_filter.field
        # this is needed to update the bound form fields with the new choices
        delattr(self, "_form")

    def remove_form_fields_that_should_be_hidden(self) -> None:
        """
        Remove form fields which should be hidden. For example facets fields with no
        facet with count > 0.
        """
        for name, post_filter in self.filters.items():
            if getattr(post_filter, "hide_form_field", False):
                del self.form.fields[name]

    @staticmethod
    def fulltext_search(queryset: PageQuerySet, _name: str, value: str) -> models.QuerySet:
        return queryset.search(value).get_queryset()



# Generated by Django 4.1.4 on 2023-02-04 14:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0034_remove_old_podcast_fields_from_blog"),
    ]

    operations = [
        migrations.RenameField(
            model_name="podcast",
            old_name="new_explicit",
            new_name="explicit",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_itunes_artwork",
            new_name="itunes_artwork",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_itunes_categories",
            new_name="itunes_categories",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_keywords",
            new_name="keywords",
        ),
    ]



from django.contrib.auth import get_user_model
from django.db import models
from model_utils.models import TimeStampedModel


class File(TimeStampedModel):
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    original = models.FileField(upload_to="cast_files/")

    def get_all_paths(self) -> set[str]:
        paths = set()
        paths.add(self.original.name)
        return paths



import pytest

from cast.models import Gallery, get_or_create_gallery, sync_media_ids


@pytest.mark.parametrize(
    "from_database, from_body, expected_to_add, expected_to_remove",
    [
        # from_database, from_body, expected_to_add, expected_to_remove
        ({}, {}, {}, {}),
        ({}, {"video": {1}}, {"video": {1}}, {}),  # add video
        ({"video": {1}}, {}, {}, {"video": {1}}),  # remove video
        ({"video": {1}}, {"video": {2}}, {"video": {2}}, {"video": {1}}),  # add video 2, remove video 1
        # add + remove different media types
        (
            {"audio": {0}, "video": {1}},
            {"video": {2}, "audio": {1}},
            {"audio": {1}, "video": {2}},
            {"video": {1}, "audio": {0}},
        ),
    ],
)
def test_sync_media_ids(from_database, from_body, expected_to_add, expected_to_remove):
    assert sync_media_ids(from_database, from_body) == (expected_to_add, expected_to_remove)


@pytest.mark.django_db()
def test_post_media_sync(post_with_gallery, python_body, body):
    post = post_with_gallery

    # make sure gallery from body was added to post.galleries
    gallery_id = list(post.media_ids_from_body["gallery"])[0]
    gallery_ids_in_db = {g.id for g in post.galleries.all()}
    assert gallery_id in gallery_ids_in_db

    # make sure removing gallery from body removes it from db
    post.body = body
    post.save()
    gallery_ids_in_db = {g.id for g in post.galleries.all()}
    assert gallery_id not in gallery_ids_in_db


def test_get_or_create_gallery_empty_image_ids():
    assert get_or_create_gallery([]) is None


@pytest.mark.django_db()
def test_get_or_create_gallery_invalid_image_id():
    gallery = get_or_create_gallery(
        [
            0,
        ]
    )
    assert gallery is None


@pytest.mark.django_db()
def test_get_or_create_gallery_new_gallery(image):
    image_ids = [
        image.pk,
    ]
    actual_gallery = get_or_create_gallery(image_ids)
    expected_gallery = Gallery.objects.filter(images__in=[image]).first()
    assert actual_gallery == expected_gallery
    assert image_ids == [i.id for i in expected_gallery.images.all()]


@pytest.mark.django_db()
def test_get_or_create_gallery_get_already_existing(image):
    image_ids = [
        image.pk,
    ]
    expected_gallery = Gallery.objects.create()
    expected_gallery.images.add(*image_ids)
    actual_gallery = get_or_create_gallery(image_ids)
    assert actual_gallery == expected_gallery



import pytest


class TestPostWithImageDetail:
    pytestmark = pytest.mark.django_db

    def test_get_post_with_image_detail(self, client, post_with_image):
        post = post_with_image
        detail_url = post.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "html" in content

        # make sure css for image included in rendered image block
        assert "cast-image" in content



# flake8: noqa:E501

from cast.models import get_video_dimensions


class TestVideoDimensions:
    def test_video_from_ios_device_portrait(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/57304811027__F8E67B21-7B33-4C70-842D-D7E708DCA683.MOV':
  Metadata:
    major_brand     : qt
    minor_version   : 0
    compatible_brands: qt
    creation_time   : 2019-02-28T12:01:50.000000Z
    com.apple.quicktime.make: Apple
    com.apple.quicktime.model: iPhone XS
    com.apple.quicktime.software: 12.1.4
    com.apple.quicktime.creationdate: 2019-02-28T13:01:50+0100
  Duration: 00:00:04.90, start: 0.000000, bitrate: 908 kb/s
    Stream #0:0(und): Video: h264 (Baseline) (avc1 / 0x31637661), yuv420p(tv, smpte170m/bt709/bt709), 480x360, 778 kb/s, 30.01 fps, 30 tbr, 600 tbn, 1200 tbc (default)
    Metadata:
      rotate          : 90
      creation_time   : 2019-02-28T12:01:50.000000Z
      handler_name    : Core Media Video
      encoder         : H.264
    Side data:
      displaymatrix: rotation of -90.00 degrees
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, mono, fltp, 91 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T12:01:50.000000Z
      handler_name    : Core Media Audio
    Stream #0:2(und): Data: none (mebx / 0x7862656D), 23 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T12:01:50.000000Z
      handler_name    : Core Media Metadata
    Stream #0:3(und): Data: none (mebx / 0x7862656D), 0 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T12:01:50.000000Z
      handler_name    : Core Media Metadata
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        print("width x height: ", width, height)
        assert width == 360
        assert height == 480

    def test_video_from_ios_device_landscape(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/57305469432__3391A666-61B4-4EEA-9373-119115C8DF9F.MOV':
  Metadata:
    major_brand     : qt
    minor_version   : 0
    compatible_brands: qt
    creation_time   : 2019-02-28T13:51:35.000000Z
    com.apple.quicktime.make: Apple
    com.apple.quicktime.model: iPhone XS
    com.apple.quicktime.software: 12.1.4
    com.apple.quicktime.creationdate: 2019-02-28T14:51:34+0100
  Duration: 00:00:02.73, start: 0.000000, bitrate: 1087 kb/s
    Stream #0:0(und): Video: h264 (Baseline) (avc1 / 0x31637661), yuv420p(tv, smpte170m/bt709/bt709), 480x360, 977 kb/s, 30 fps, 30 tbr, 600 tbn, 1200 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T13:51:35.000000Z
      handler_name    : Core Media Video
      encoder         : H.264
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, mono, fltp, 88 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:51:35.000000Z
      handler_name    : Core Media Audio
    Stream #0:2(und): Data: none (mebx / 0x7862656D), 0 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:51:35.000000Z
      handler_name    : Core Media Metadata
    Stream #0:3(und): Data: none (mebx / 0x7862656D), 0 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:51:35.000000Z
      handler_name    : Core Media Metadata
Stream mapping:
  Stream #0:0 -> #0:0 (h264 (native) -> mjpeg (native))
Press [q] to stop, [?] for help
[swscaler @ 0x7fe8dd0c7600] deprecated pixel format used, make sure you did set range correctly
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_p5k4i2f3.jpg':
  Metadata:
    major_brand     : qt
    minor_version   : 0
    compatible_brands: qt
    com.apple.quicktime.creationdate: 2019-02-28T14:51:34+0100
    com.apple.quicktime.make: Apple
    com.apple.quicktime.model: iPhone XS
    com.apple.quicktime.software: 12.1.4
    encoder         : Lavf58.20.100
    Stream #0:0(und): Video: mjpeg, yuvj420p(pc), 480x360, q=2-31, 200 kb/s, 30 fps, 30 tbn, 30 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T13:51:35.000000Z
      handler_name    : Core Media Video
      encoder         : Lavc58.35.100 mjpeg
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
frame=    1 fps=0.0 q=6.2 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=2.47x
video:22kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        print("width x height: ", width, height)
        assert width == 480
        assert height == 360

    def test_video_from_ios_portrait(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/E6B2E327-98BB-4348-8606-1E531D121BB3.MOV':
  Metadata:
    major_brand     : qt
    minor_version   : 0
    compatible_brands: qt
    creation_time   : 2019-02-28T13:55:07.000000Z
    com.apple.quicktime.location.ISO6709: +51.2382+006.7899+036.689/
    com.apple.quicktime.make: Apple
    com.apple.quicktime.model: iPhone XS
    com.apple.quicktime.software: 12.1.4
    com.apple.quicktime.creationdate: 2019-02-28T14:55:06+0100
  Duration: 00:00:03.57, start: 0.000000, bitrate: 28918 kb/s
    Stream #0:0(und): Video: hevc (Main) (hvc1 / 0x31637668), yuv420p(tv, bt709), 3840x2160, 28726 kb/s, 29.97 fps, 29.97 tbr, 600 tbn, 600 tbc (default)
    Metadata:
      rotate          : 90
      creation_time   : 2019-02-28T13:55:07.000000Z
      handler_name    : Core Media Video
      encoder         : HEVC
    Side data:
      displaymatrix: rotation of -90.00 degrees
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 168 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:55:07.000000Z
      handler_name    : Core Media Audio
    Stream #0:2(und): Data: none (mebx / 0x7862656D), 0 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:55:07.000000Z
      handler_name    : Core Media Metadata
    Stream #0:3(und): Data: none (mebx / 0x7862656D), 0 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:55:07.000000Z
      handler_name    : Core Media Metadata
Stream mapping:
  Stream #0:0 -> #0:0 (hevc (native) -> mjpeg (native))
Press [q] to stop, [?] for help
[swscaler @ 0x7fdc29801000] deprecated pixel format used, make sure you did set range correctly
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_3w_mupml.jpg':
  Metadata:
    major_brand     : qt
    minor_version   : 0
    compatible_brands: qt
    com.apple.quicktime.creationdate: 2019-02-28T14:55:06+0100
    com.apple.quicktime.location.ISO6709: +51.2382+006.7899+036.689/
    com.apple.quicktime.make: Apple
    com.apple.quicktime.model: iPhone XS
    com.apple.quicktime.software: 12.1.4
    encoder         : Lavf58.20.100
    Stream #0:0(und): Video: mjpeg, yuvj420p(pc), 2160x3840, q=2-31, 200 kb/s, 29.97 fps, 29.97 tbn, 29.97 tbc (default)
    Metadata:
      encoder         : Lavc58.35.100 mjpeg
      creation_time   : 2019-02-28T13:55:07.000000Z
      handler_name    : Core Media Video
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
      displaymatrix: rotation of -0.00 degrees
frame=    1 fps=0.0 q=8.8 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=0.0517x
video:211kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        print("width x height: ", width, height)
        assert width == 2160
        assert height == 3840

    def test_video_from_android_portrait(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/VID_20190228_144156.mp4':
  Metadata:
    major_brand     : mp42
    minor_version   : 0
    compatible_brands: isommp42
    creation_time   : 2019-02-28T13:41:56.000000Z
    location        : +51.2380+006.7902/
    location-eng    : +51.2380+006.7902/
    com.android.version: 9
  Duration: 00:00:04.06, start: 0.000000, bitrate: 21343 kb/s
    Stream #0:0(eng): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 190 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T13:41:56.000000Z
      handler_name    : SoundHandle
    Stream #0:1(eng): Video: h264 (High) (avc1 / 0x31637661), yuvj420p(pc, bt470bg/bt470bg/smpte170m), 1920x1080, 21917 kb/s, SAR 1:1 DAR 16:9, 29.88 fps, 30.13 tbr, 90k tbn, 180k tbc (default)
    Metadata:
      rotate          : 270
      creation_time   : 2019-02-28T13:41:56.000000Z
      handler_name    : VideoHandle
    Side data:
      displaymatrix: rotation of 90.00 degrees
Stream mapping:
  Stream #0:1 -> #0:0 (h264 (native) -> mjpeg (native))
Press [q] to stop, [?] for help
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_22_kvc0v.jpg':
  Metadata:
    major_brand     : mp42
    minor_version   : 0
    compatible_brands: isommp42
    com.android.version: 9
    location        : +51.2380+006.7902/
    location-eng    : +51.2380+006.7902/
    encoder         : Lavf58.20.100
    Stream #0:0(eng): Video: mjpeg, yuvj420p(pc), 1080x1920 [SAR 1:1 DAR 9:16], q=2-31, 200 kb/s, 30.13 fps, 30.13 tbn, 30.13 tbc (default)
    Metadata:
      encoder         : Lavc58.35.100 mjpeg
      creation_time   : 2019-02-28T13:41:56.000000Z
      handler_name    : VideoHandle
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
      displaymatrix: rotation of -0.00 degrees
frame=    1 fps=0.0 q=6.4 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=0.263x
video:97kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width == 1080
        assert height == 1920

    def test_video_from_android_landscape(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/VID_20190228_150527.mp4':
  Metadata:
    major_brand     : mp42
    minor_version   : 0
    compatible_brands: isommp42
    creation_time   : 2019-02-28T14:05:27.000000Z
    location        : +51.2380+006.7902/
    location-eng    : +51.2380+006.7902/
    com.android.version: 9
  Duration: 00:00:02.23, start: 0.000000, bitrate: 22175 kb/s
    Stream #0:0(eng): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 193 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T14:05:27.000000Z
      handler_name    : SoundHandle
    Stream #0:1(eng): Video: h264 (High) (avc1 / 0x31637661), yuvj420p(pc, bt470bg/bt470bg/smpte170m), 1920x1080, 21968 kb/s, SAR 1:1 DAR 16:9, 30.01 fps, 30 tbr, 90k tbn, 180k tbc (default)
    Metadata:
      creation_time   : 2019-02-28T14:05:27.000000Z
      handler_name    : VideoHandle
Stream mapping:
  Stream #0:1 -> #0:0 (h264 (native) -> mjpeg (native))
Press [q] to stop, [?] for help
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_qcptirnt.jpg':
  Metadata:
    major_brand     : mp42
    minor_version   : 0
    compatible_brands: isommp42
    com.android.version: 9
    location        : +51.2380+006.7902/
    location-eng    : +51.2380+006.7902/
    encoder         : Lavf58.20.100
    Stream #0:0(eng): Video: mjpeg, yuvj420p(pc), 1920x1080 [SAR 1:1 DAR 16:9], q=2-31, 200 kb/s, 30 fps, 30 tbn, 30 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T14:05:27.000000Z
      handler_name    : VideoHandle
      encoder         : Lavc58.35.100 mjpeg
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
frame=    1 fps=0.0 q=6.4 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=0.332x
video:75kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width == 1920
        assert height == 1080

    def test_video_from_handbrake_landscape(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/IMG_0563_kqTcRSQ.mp4':
  Metadata:
    major_brand     : mp42
    minor_version   : 512
    compatible_brands: isomiso2avc1mp41
    creation_time   : 2019-02-28T11:33:03.000000Z
    title           : Ein langes Brillenvideo
    date            : 2018-08-21T12:28:17+0200
    encoder         : HandBrake 1.2.1 2019021700
  Duration: 00:04:32.19, start: 0.000000, bitrate: 3104 kb/s
    Stream #0:0(und): Video: h264 (Main) (avc1 / 0x31637661), yuv420p(tv, bt709), 1920x1080 [SAR 1:1 DAR 16:9], 2931 kb/s, 30 fps, 30 tbr, 90k tbn, 180k tbc (default)
    Metadata:
      creation_time   : 2019-02-28T11:33:03.000000Z
      handler_name    : VideoHandler
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, mono, fltp, 164 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T11:33:03.000000Z
      handler_name    : SoundHandler
Stream mapping:
  Stream #0:0 -> #0:0 (h264 (native) -> mjpeg (native))
Press [q] to stop, [?] for help
[swscaler @ 0x7f8a4513ac00] deprecated pixel format used, make sure you did set range correctly
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_abfyafdr.jpg':
  Metadata:
    major_brand     : mp42
    minor_version   : 512
    compatible_brands: isomiso2avc1mp41
    date            : 2018-08-21T12:28:17+0200
    title           : Ein langes Brillenvideo
    encoder         : Lavf58.20.100
    Stream #0:0(und): Video: mjpeg, yuvj420p(pc), 1920x1080 [SAR 1:1 DAR 16:9], q=2-31, 200 kb/s, 30 fps, 30 tbn, 30 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T11:33:03.000000Z
      handler_name    : VideoHandler
      encoder         : Lavc58.35.100 mjpeg
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
frame=    1 fps=0.0 q=8.6 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=0.194x
video:84kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width == 1920
        assert height == 1080

    def test_video_from_ios_fotos_landscape(self):
        ffmpeg_output = """
ffmpeg version 4.1.1 Copyright (c) 2000-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.0 (clang-1000.11.45.5)
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from '/Users/jochen/projects/homepage/homepage/media/cast_videos/IMG_0563.m4v':
  Metadata:
    major_brand     : M4V
    minor_version   : 1
    compatible_brands: M4V M4A mp42isom
    creation_time   : 2019-02-28T11:29:47.000000Z
    date            : 2018-08-21T12:28:17+0200
    title           : Ein langes Brillenvideo
    make            : Apple
  Duration: 00:04:32.11, start: 0.000000, bitrate: 10591 kb/s
    Stream #0:0(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, mono, fltp, 88 kb/s (default)
    Metadata:
      creation_time   : 2019-02-28T11:29:47.000000Z
      handler_name    : Core Media Audio
    Stream #0:1(und): Video: h264 (High) (avc1 / 0x31637661), yuv420p(tv, bt709), 1920x1080 [SAR 1:1 DAR 16:9], 10497 kb/s, 30.01 fps, 30 tbr, 600 tbn, 1200 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T11:29:47.000000Z
      handler_name    : Core Media Video
Stream mapping:
  Stream #0:1 -> #0:0 (h264 (native) -> mjpeg (native))
Press [q] to stop, [?] for help
[swscaler @ 0x7f8115882e00] deprecated pixel format used, make sure you did set range correctly
Output #0, image2, to '/var/folders/yq/lq6vnk9s693bp4xr5wktm1vh0000gn/T/poster_5rsf6f9y.jpg':
  Metadata:
    major_brand     : M4V
    minor_version   : 1
    compatible_brands: M4V M4A mp42isom
    make            : Apple
    date            : 2018-08-21T12:28:17+0200
    title           : Ein langes Brillenvideo
    encoder         : Lavf58.20.100
    Stream #0:0(und): Video: mjpeg, yuvj420p(pc), 1920x1080 [SAR 1:1 DAR 16:9], q=2-31, 200 kb/s, 30 fps, 30 tbn, 30 tbc (default)
    Metadata:
      creation_time   : 2019-02-28T11:29:47.000000Z
      handler_name    : Core Media Video
      encoder         : Lavc58.35.100 mjpeg
    Side data:
      cpb: bitrate max/min/avg: 0/0/200000 buffer size: 0 vbv_delay: -1
frame=    1 fps=0.0 q=8.6 Lsize=N/A time=00:00:00.03 bitrate=N/A speed=0.269x
video:91kB audio:0kB subtitle:0kB other streams:0kB global headers:0kB muxing overhead: unknown
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width == 1920
        assert height == 1080

    def test_video_from_handbrake_portrait(self):
        ffmpeg_output = """
ffprobe version 4.1.3 Copyright (c) 2007-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.1 (clang-1001.0.46.4)
  configuration: --prefix=/usr/local/Cellar/ffmpeg/4.1.3_1 --enable-shared --enable-pthreads --enable-version3 --enable-hardcoded-tables --enable-avresample --cc=clang --host-cflags='-I/Library/Java/JavaVirtualMachines/adoptopenjdk-11.0.2.jdk/Contents/Home/include -I/Library/Java/JavaVirtualMachines/adoptopenjdk-11.0.2.jdk/Contents/Home/include/darwin' --host-ldflags= --enable-ffplay --enable-gnutls --enable-gpl --enable-libaom --enable-libbluray --enable-libmp3lame --enable-libopus --enable-librubberband --enable-libsnappy --enable-libtesseract --enable-libtheora --enable-libvorbis --enable-libvpx --enable-libx264 --enable-libx265 --enable-libxvid --enable-lzma --enable-libfontconfig --enable-libfreetype --enable-frei0r --enable-libass --enable-libopencore-amrnb --enable-libopencore-amrwb --enable-libopenjpeg --enable-librtmp --enable-libspeex --enable-videotoolbox --disable-libjack --disable-indev=jack --enable-libaom --enable-libsoxr
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from 'Michaelsbergschaukel.mp4':
  Metadata:
    major_brand     : mp42
    minor_version   : 512
    compatible_brands: isomiso2avc1mp41
    creation_time   : 2019-04-27T00:04:26.000000Z
    title           : Michaelsbergschaukel
    date            : 2018-09-22T14:58:10+0200
    encoder         : HandBrake 1.2.1 2019021700
  Duration: 00:00:13.79, start: 0.000000, bitrate: 10086 kb/s
    Stream #0:0(und): Video: h264 (Main) (avc1 / 0x31637661), yuv420p(tv, bt709), 1920x1080 [SAR 81:256 DAR 9:16], 9970 kb/s, 29.97 fps, 29.97 tbr, 90k tbn, 180k tbc (default)
    Metadata:
      creation_time   : 2019-04-27T00:04:26.000000Z
      handler_name    : VideoHandler
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 162 kb/s (default)
    Metadata:
      creation_time   : 2019-04-27T00:04:26.000000Z
      handler_name    : SoundHandler
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        print("width x height: ", width, height)
        assert width == 1080
        assert height == 1920

    def test_video_from_handbrake_landscape_2(self):
        ffmpeg_output = """
ffprobe version 4.1.3 Copyright (c) 2007-2019 the FFmpeg developers
  built with Apple LLVM version 10.0.1 (clang-1001.0.46.4)
  configuration: --prefix=/usr/local/Cellar/ffmpeg/4.1.3_1 --enable-shared --enable-pthreads --enable-version3 --enable-hardcoded-tables --enable-avresample --cc=clang --host-cflags='-I/Library/Java/JavaVirtualMachines/adoptopenjdk-11.0.2.jdk/Contents/Home/include -I/Library/Java/JavaVirtualMachines/adoptopenjdk-11.0.2.jdk/Contents/Home/include/darwin' --host-ldflags= --enable-ffplay --enable-gnutls --enable-gpl --enable-libaom --enable-libbluray --enable-libmp3lame --enable-libopus --enable-librubberband --enable-libsnappy --enable-libtesseract --enable-libtheora --enable-libvorbis --enable-libvpx --enable-libx264 --enable-libx265 --enable-libxvid --enable-lzma --enable-libfontconfig --enable-libfreetype --enable-frei0r --enable-libass --enable-libopencore-amrnb --enable-libopencore-amrwb --enable-libopenjpeg --enable-librtmp --enable-libspeex --enable-videotoolbox --disable-libjack --disable-indev=jack --enable-libaom --enable-libsoxr
  libavutil      56. 22.100 / 56. 22.100
  libavcodec     58. 35.100 / 58. 35.100
  libavformat    58. 20.100 / 58. 20.100
  libavdevice    58.  5.100 / 58.  5.100
  libavfilter     7. 40.101 /  7. 40.101
  libavresample   4.  0.  0 /  4.  0.  0
  libswscale      5.  3.100 /  5.  3.100
  libswresample   3.  3.100 /  3.  3.100
  libpostproc    55.  3.100 / 55.  3.100
Input #0, mov,mp4,m4a,3gp,3g2,mj2, from 'Tripptrappkuckuck.mp4':
  Metadata:
    major_brand     : mp42
    minor_version   : 512
    compatible_brands: isomiso2avc1mp41
    creation_time   : 2019-04-27T00:12:09.000000Z
    title           : Tripptrappkuckuck
    date            : 2018-09-25T21:25:49+0200
    encoder         : HandBrake 1.2.1 2019021700
  Duration: 00:00:41.82, start: 0.000000, bitrate: 2030 kb/s
    Stream #0:0(und): Video: h264 (Main) (avc1 / 0x31637661), yuv420p(tv, bt709), 1920x1080 [SAR 1:1 DAR 16:9], 1860 kb/s, 29.97 fps, 29.97 tbr, 90k tbn, 180k tbc (default)
    Metadata:
      creation_time   : 2019-04-27T00:12:09.000000Z
      handler_name    : VideoHandler
    Stream #0:1(und): Audio: aac (LC) (mp4a / 0x6134706D), 44100 Hz, stereo, fltp, 162 kb/s (default)
    Metadata:
      creation_time   : 2019-04-27T00:12:09.000000Z
      handler_name    : SoundHandler
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width == 1920
        assert height == 1080

    def test_video_from_empty(self):
        ffmpeg_output = """
           foo bar baz
        """
        width, height = get_video_dimensions(ffmpeg_output.split("\n"))
        assert width is None
        assert height is None



from typing import Any

from django.urls import path

from ..views import video as views

urlpatterns: list[Any] = [
    path("video", views.index, name="index"),
    path(r"video/add/", views.add, name="add"),
    path("video/edit/<int:video_id>/", views.edit, name="edit"),
    path("video/delete/<int:video_id>/", views.delete, name="delete"),
    path("video/chooser/", views.chooser, name="chooser"),
    path("video/chooser/upload/", views.chooser_upload, name="chooser_upload"),
    path("video/chooser/<int:video_id>/", views.chosen, name="chosen"),
]



# Generated by Django 4.1.5 on 2023-01-21 06:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0027_episode"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="post",
            name="block",
        ),
        migrations.RemoveField(
            model_name="post",
            name="explicit",
        ),
        migrations.RemoveField(
            model_name="post",
            name="keywords",
        ),
        migrations.RemoveField(
            model_name="post",
            name="podcast_audio",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_block",
            new_name="block",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_explicit",
            new_name="explicit",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_keywords",
            new_name="keywords",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_podcast_audio",
            new_name="podcast_audio",
        ),
    ]



from django.views import csrf, defaults

from cast.models import TemplateBaseDirectory


def get_template_base_directory(request):
    """Get the template base directory for the current request, ignoring all exceptions"""
    try:
        return TemplateBaseDirectory.for_request(request).name
    except Exception:
        # If we can't get the template base directory from the database,
        # just use the default one.
        return "plain"


def get_template_name(request, base_template_name):
    """Get the template name for the current request"""
    template_base_dir = get_template_base_directory(request)
    return f"cast/{template_base_dir}/{base_template_name}"


def page_not_found(request, exception):
    """Just call the default page_not_found view, but with a custom template"""
    return defaults.page_not_found(request, exception, template_name=get_template_name(request, "404.html"))


def server_error(request):
    """Just call the default server_error view, but with a custom template"""
    return defaults.server_error(request, template_name=get_template_name(request, "500.html"))


def bad_request(request, exception):
    """Just call the default bad_request view, but with a custom template"""
    return defaults.bad_request(request, exception, template_name=get_template_name(request, "400.html"))


def permission_denied(request, exception):
    """Just call the default permission_denied view, but with a custom template"""
    return defaults.permission_denied(request, exception, template_name=get_template_name(request, "403.html"))


def csrf_failure(request, reason=""):
    """Just call the default csrf_failure view, but with a custom template"""
    return csrf.csrf_failure(request, reason, template_name=get_template_name(request, "403_csrf.html"))



from typing import Any

from django.urls import include, path, re_path
from rest_framework.schemas import get_schema_view

from . import views

app_name = "api"
schema_view = get_schema_view(title="Cast API")

urlpatterns: list[Any] = [
    path("schema/", schema_view),
    path("", views.api_root, name="root"),
    # video
    path("videos/", views.VideoListView.as_view(), name="video_list"),
    re_path(r"^videos/(?P<pk>\d+)/?$", views.VideoDetailView.as_view(), name="video_detail"),
    path(
        "upload_video/",
        views.VideoCreateView.as_view(),
        name="upload_video",
    ),
    # audio
    path("audios/", views.AudioListView.as_view(), name="audio_list"),
    re_path(r"^audios/(?P<pk>\d+)/?$", views.AudioDetailView.as_view(), name="audio_detail"),
    re_path(
        r"^audios/podlove/(?P<pk>\d+)/?$",
        views.AudioPodloveDetailView.as_view(),
        name="audio_podlove_detail",
    ),
    # facet counts
    path("facet_counts/", views.FacetCountListView.as_view(), name="facet-counts-list"),
    re_path(r"facet_counts/(?P<pk>\d+)/?$", views.FacetCountsDetailView.as_view(), name="facet-counts-detail"),
    # comment training data
    path("comment_training_data/", views.CommentTrainingDataView.as_view(), name="comment-training-data"),
    # themes
    path("themes/", views.ThemeListView.as_view(), name="theme-list"),
    path("update_theme/", views.UpdateThemeView.as_view(), name="theme-update"),
    # wagtail api
    path("wagtail/", include((views.wagtail_api_router.get_urlpatterns(), "api"), namespace="wagtail")),
]



import pytest
from django.core.paginator import Paginator
from django.http import QueryDict
from django.shortcuts import render

from cast.models import Blog
from cast.views.wagtail_pagination import paginate


def test_pagination_template_is_not_paginated(simple_request):
    r = render(simple_request, "cast/plain/pagination.html", {})
    html = r.content.decode("utf-8").strip()
    assert html == ""


def test_pagination_template_is_paginated(simple_request):
    r = render(simple_request, "cast/plain/pagination.html", {"is_paginated": True})
    html = r.content.decode("utf-8").strip()
    assert "pagination" in html


def test_pagination_template_is_paginated_long(simple_request):
    paginator = Paginator(range(1000), 2)
    page = paginator.page(9)
    context = {
        "is_paginated": page.has_other_pages(),
        "paginator": paginator,
        "page_obj": page,
        "object_list": page.object_list,
        "page_range": page.paginator.get_elided_page_range(page.number, on_each_side=2, on_ends=1),
    }
    r = render(simple_request, "cast/plain/pagination.html", context)
    html = r.content.decode("utf-8").strip()
    assert "page=1" in html  # first page
    assert "page=500" in html  # last page

    # two ellipsis in the middle which are not links
    ellipsis_items = [line for line in html.splitlines() if "…" in line]
    assert len(ellipsis_items) == 2
    assert (
        '<span class="cast-page-link-disabled">…</span>' == ellipsis_items[0].strip()
    )  # ellipsis in the middle which is not a link


@pytest.mark.parametrize(
    "query_string, expected_other_get_params",
    [
        ("", ""),
        ("foo=bar&bar=foo&page=3", "&foo=bar&bar=foo"),
    ],
)
def test_get_other_get_params(query_string, expected_other_get_params):
    assert Blog.get_other_get_params(QueryDict(query_string)) == expected_other_get_params


def test_wagtail_admin_pagination_starts_with_first_page(simple_request):
    """
    Test that the pagination starts with the first page. There was a bug
    when the page was fetched like this:
        page = paginator.get_page(request.GET.get(page_key, 0))

    This would return the last page if there was more than one page. Make
    sure this does not happen again.
    """
    paginator, page = paginate(simple_request, range(100))  # noqa
    assert page.number == 1



# Generated by Django 5.0 on 2023-12-22 22:25

import cast.blocks
import wagtail.blocks
import wagtail.embeds.blocks
import wagtail.fields
import wagtail.images.blocks
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0051_use_own_image_chooser_block"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blog",
            name="template_base_dir",
            field=models.CharField(
                blank=True,
                choices=[
                    ("bootstrap4", "Bootstrap 4"),
                    ("plain", "Just HTML"),
                    ("vue", "Vue.js"),
                    ("bootstrap5", "Bootstrap 5"),
                ],
                default=None,
                help_text="The theme to use for this blog implemented as a template base directory. If not set, the template base directory will be determined by a site setting.",
                max_length=128,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="post",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    (
                        "overview",
                        wagtail.blocks.StreamBlock(
                            [
                                (
                                    "heading",
                                    wagtail.blocks.CharBlock(
                                        form_classname="full title"
                                    ),
                                ),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                (
                                    "code",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "language",
                                                wagtail.blocks.CharBlock(
                                                    help_text="The language of the code block"
                                                ),
                                            ),
                                            (
                                                "source",
                                                wagtail.blocks.TextBlock(
                                                    help_text="The source code of the block",
                                                    rows=8,
                                                ),
                                            ),
                                        ],
                                        icon="code",
                                    ),
                                ),
                                (
                                    "image",
                                    cast.blocks.CastImageChooserBlock(
                                        template="cast/image/image.html"
                                    ),
                                ),
                                (
                                    "gallery",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "gallery",
                                                cast.blocks.GalleryBlock(
                                                    wagtail.images.blocks.ImageChooserBlock()
                                                ),
                                            ),
                                            (
                                                "layout",
                                                wagtail.blocks.ChoiceBlock(
                                                    choices=[
                                                        (
                                                            "web_component",
                                                            "Web Component with Modal",
                                                        ),
                                                        ("htmx", "HTMX based layout"),
                                                    ]
                                                ),
                                            ),
                                        ]
                                    ),
                                ),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(
                                        icon="media", template="cast/video/video.html"
                                    ),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(
                                        icon="media", template="cast/audio/audio.html"
                                    ),
                                ),
                            ]
                        ),
                    ),
                    (
                        "detail",
                        wagtail.blocks.StreamBlock(
                            [
                                (
                                    "heading",
                                    wagtail.blocks.CharBlock(
                                        form_classname="full title"
                                    ),
                                ),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                (
                                    "code",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "language",
                                                wagtail.blocks.CharBlock(
                                                    help_text="The language of the code block"
                                                ),
                                            ),
                                            (
                                                "source",
                                                wagtail.blocks.TextBlock(
                                                    help_text="The source code of the block",
                                                    rows=8,
                                                ),
                                            ),
                                        ],
                                        icon="code",
                                    ),
                                ),
                                (
                                    "image",
                                    cast.blocks.CastImageChooserBlock(
                                        template="cast/image/image.html"
                                    ),
                                ),
                                (
                                    "gallery",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "gallery",
                                                cast.blocks.GalleryBlock(
                                                    wagtail.images.blocks.ImageChooserBlock()
                                                ),
                                            ),
                                            (
                                                "layout",
                                                wagtail.blocks.ChoiceBlock(
                                                    choices=[
                                                        (
                                                            "web_component",
                                                            "Web Component with Modal",
                                                        ),
                                                        ("htmx", "HTMX based layout"),
                                                    ]
                                                ),
                                            ),
                                        ]
                                    ),
                                ),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(
                                        icon="media", template="cast/video/video.html"
                                    ),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(
                                        icon="media", template="cast/audio/audio.html"
                                    ),
                                ),
                            ]
                        ),
                    ),
                ],
                use_json_field=True,
            ),
        ),
        migrations.AlterField(
            model_name="templatebasedirectory",
            name="name",
            field=models.CharField(
                choices=[
                    ("bootstrap4", "Bootstrap 4"),
                    ("plain", "Just HTML"),
                    ("vue", "Vue.js"),
                    ("bootstrap5", "Bootstrap 5"),
                ],
                default="bootstrap4",
                help_text="The theme to use for this site implemented as a template base directory. It's possible to overwrite this setting for each blog.If you want to use a custom theme, you have to create a new directory in your template directory named cast/<your-theme-name>/ and put all required templates in there.",
                max_length=128,
            ),
        ),
    ]



from unittest.mock import patch

import pytest
from django.urls import reverse

from cast.models import Video


class TestPostWithVideoDetail:
    pytestmark = pytest.mark.django_db

    def test_get_post_with_video_detail(self, client, post_with_video):
        video = post_with_video.videos.first()
        detail_url = post_with_video.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "html" in content

        # make sure video is included in rendered video block
        assert "block-video" in content
        assert video.filename in content


def get_endpoint_urls_without_args():
    urls = {}
    view_names = ["index", "add", "chooser", "chooser_upload"]
    for view_name in view_names:
        urls[view_name] = reverse(f"castvideo:{view_name}")
    return urls


def get_endpoint_urls_with_args(video):
    urls = {}
    view_names = ["edit", "delete", "chosen"]
    for view_name in view_names:
        urls[view_name] = reverse(f"castvideo:{view_name}", args=(video.id,))
    return urls


class VideoUrls:
    def __init__(self, video):
        self.video = video
        self.urls = get_endpoint_urls_without_args()
        self.urls.update(get_endpoint_urls_with_args(video))

    def __getattr__(self, item):
        return self.urls[item]


@pytest.fixture
def video_urls(video):
    return VideoUrls(video)


class TestAllVideoEndpoints:
    pytestmark = pytest.mark.django_db

    def test_get_all_not_authenticated(self, client, video_urls):
        for view_name, url in video_urls.urls.items():
            r = client.get(url)

            # redirect to login
            assert r.status_code == 302
            login_url = reverse("wagtailadmin_login")
            assert login_url in r.url

    def test_get_all_authenticated(self, authenticated_client, video_urls):
        for view_name, url in video_urls.urls.items():
            r = authenticated_client.get(url)

            # assert we are not redirected to login
            assert r.status_code == 200


class TestVideoIndex:
    pytestmark = pytest.mark.django_db

    def test_get_index(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.index)

        assert r.status_code == 200
        content = r.content.decode("utf-8")

        # make sure it's the media results page
        assert "html" in content
        assert "media-results" in content

        # make sure video_urls.video is included in results
        assert video_urls.video.title in content

    def test_get_index_ajax(self, authenticated_client, video_urls):
        headers = {"HTTP_X_REQUESTED_WITH": "XMLHttpRequest"}
        r = authenticated_client.get(video_urls.index, **headers)

        assert r.status_code == 200
        content = r.content.decode("utf-8")

        # make sure it's the media results page
        assert "table" in content
        assert "listing" in content

        # make sure video_urls.video is included in results
        assert video_urls.video.title in content

    def test_get_index_with_search(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.index, {"q": video_urls.video.title})

        assert r.status_code == 200
        content = r.content.decode("utf-8")

        # make sure it's the media results page
        assert "html" in content
        assert "media-results" in content

        # make sure video_urls.video is included in results
        assert video_urls.video.title in content

    def test_get_index_with_search_invalid(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.index, {"q": " "})

        assert r.status_code == 200
        content = r.content.decode("utf-8")

        # make sure it's the media results page
        assert "html" in content
        assert "media-results" in content

        # make sure video_urls.video is included in results
        print(content)
        assert video_urls.video.title in content

    def test_get_index_with_pagination(self, authenticated_client, user):
        video_models = []
        for i in range(1, 3):
            video = Video(user=user, title=f"video {i}")
            video.save()
            video_models.append(video)
        index_url = reverse("castvideo:index")
        with patch("cast.views.video.MENU_ITEM_PAGINATION", return_value=1):
            r = authenticated_client.get(index_url, {"p": "1"})
        videos = r.context["videos"]

        # make sure we got last video from first page
        assert len(videos) == 1
        assert videos[0] == video_models[-1]

        with patch("cast.views.video.MENU_ITEM_PAGINATION", return_value=1):
            r = authenticated_client.get(index_url, {"p": "2"})
        videos = r.context["videos"]

        # make sure we got first video from last page
        assert len(videos) == 1
        assert videos[0] == video_models[0]


class TestVideoAdd:
    pytestmark = pytest.mark.django_db

    def test_get_add_video(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.add)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert "Uploading…" in content

    def test_post_add_video_invalid_form(self, authenticated_client):
        add_url = reverse("castvideo:add")

        post_data = {
            "title": "foobar",
            "tags": "foo,bar,baz",
        }
        r = authenticated_client.post(add_url, post_data)

        # make sure we dont get redirected to index
        assert r.status_code == 200

        # make sure we didn't create a video
        Video.objects.first() is None

    def test_post_add_video(self, authenticated_client, minimal_mp4):
        add_url = reverse("castvideo:add")

        post_data = {
            "title": "foobar",
            "tags": "foo,bar,baz",
            "original": minimal_mp4,
        }
        r = authenticated_client.post(add_url, post_data)

        # make sure we get redirected to index
        assert r.status_code == 302
        assert r.url == reverse("castvideo:index")

        # make sure field were saved correctly
        video = Video.objects.first()
        assert video.title == post_data["title"]

        actual_tags = {t.name for t in video.tags.all()}
        expected_tags = set(post_data["tags"].split(","))
        assert actual_tags == expected_tags


class TestVideoEdit:
    pytestmark = pytest.mark.django_db

    def test_get_edit_video(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.edit)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert "Delete" in content

    def test_get_edit_video_without_original(self, authenticated_client, video_without_original):
        video = video_without_original
        edit_url = reverse("castvideo:edit", args=(video.id,))
        r = authenticated_client.get(edit_url)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert "Delete" in content

    def test_get_edit_video_with_original_no_filesize(self, settings, authenticated_client, video_without_file):
        settings.DEFAULT_FILE_STORAGE = "django.core.files.storage.FileSystemStorage"
        video = video_without_file
        edit_url = reverse("castvideo:edit", args=(video.id,))
        r = authenticated_client.get(edit_url)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert "Delete" in content

    def test_post_edit_video_invalid_form(self, authenticated_client, video_urls):
        post_data = {"foo": "bar"}  # must not be empty because of if request.POST claus
        r = authenticated_client.post(video_urls.edit, post_data)

        # make sure we dont get redirected to index
        assert r.status_code == 200

    def test_post_edit_video_title(self, authenticated_client, video_urls):
        video = video_urls.video
        post_data = {
            "title": "changed title",
        }
        r = authenticated_client.post(video_urls.edit, post_data)

        # make sure we get redirected to index
        assert r.status_code == 302
        assert r.url == video_urls.index

        # make sure title was changes
        video.refresh_from_db()
        assert video.title == post_data["title"]

    def test_post_edit_video_original(self, authenticated_client, video_urls, minimal_mp4):
        post_data = {
            "title": "asdf",
            "original": minimal_mp4,
        }
        r = authenticated_client.post(video_urls.edit, post_data)

        # make sure we get redirected to index
        assert r.status_code == 302
        assert r.url == video_urls.index


class TestVideoDelete:
    pytestmark = pytest.mark.django_db

    def test_get_delete_video(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.delete)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert "Are you sure you want to delete this video?" in content

    def test_post_delete_video(self, authenticated_client, video_urls):
        video = video_urls.video
        # post data is necessary because of if request.POST
        r = authenticated_client.post(video_urls.delete, {"delete": "yes"})

        # make sure we get redirected to index
        assert r.status_code == 302
        assert r.url == video_urls.index

        # make sure video was deleted
        with pytest.raises(Video.DoesNotExist):
            video.refresh_from_db()


class TestVideoChosen:
    pytestmark = pytest.mark.django_db

    def test_get_chosen_video_not_found(self, authenticated_client, video_urls):
        video = video_urls.video
        video.delete()
        r = authenticated_client.get(video_urls.chosen)

        assert r.status_code == 404

    def test_get_chosen_video_success(self, authenticated_client, video_urls):
        video = video_urls.video
        r = authenticated_client.get(video_urls.chosen)

        assert r.status_code == 200

        # make sure returned data belongs to the right video instance
        data = r.json()
        assert data["result"]["title"] == video.title


class TestVideoChooser:
    pytestmark = pytest.mark.django_db

    def test_get_video_in_chooser(self, authenticated_client, video_urls):
        video = video_urls.video
        r = authenticated_client.get(video_urls.chooser)

        assert r.status_code == 200

        # make sure existing video is in chooser
        content = r.content.decode("utf-8")
        assert video.title in content

        # make sure prefix for form fields is set
        assert "media-chooser-upload" in content

    def test_get_video_chooser_with_search(self, authenticated_client, video_urls):
        r = authenticated_client.get(video_urls.chooser, {"q": video_urls.video.title})

        assert r.status_code == 200

        # make sure searched video is included in results
        assert r.context["videos"][0] == video_urls.video

    def test_get_video_chooser_with_search_invalid(self, authenticated_client, video_urls):
        # {"p": "1"} (page 1) leads to the search form being invalid
        r = authenticated_client.get(video_urls.chooser, {"p": "1"})

        assert r.status_code == 200

        # make sure searched video is included in results
        assert r.context["videos"][0] == video_urls.video

    def test_get_video_chooser_with_pagination(self, authenticated_client, user):
        video_models = []
        for i in range(1, 3):
            video = Video(user=user, title=f"video {i}")
            video.save()
            video_models.append(video)
        chooser_url = reverse("castvideo:chooser")
        with patch("cast.views.video.CHOOSER_PAGINATION", return_value=1):
            r = authenticated_client.get(chooser_url, {"p": "1"})
        videos = r.context["videos"]

        # make sure we got last video from first page
        assert len(videos) == 1
        assert videos[0] == video_models[-1]

        with patch("cast.views.video.CHOOSER_PAGINATION", return_value=1):
            r = authenticated_client.get(chooser_url, {"p": "2"})
        videos = r.context["videos"]

        # make sure we got first video from last page
        assert len(videos) == 1
        assert videos[0] == video_models[0]


class TestVideoChooserUpload:
    pytestmark = pytest.mark.django_db

    def test_get_video_in_chooser_upload(self, authenticated_client, video_urls):
        video = video_urls.video
        r = authenticated_client.get(video_urls.chooser_upload)

        assert r.status_code == 200
        content = r.content.decode("utf-8")
        assert video.title in content

    def test_post_upload_video_form_invalid(self, authenticated_client):
        upload_url = reverse("castvideo:chooser_upload")
        post_data = {"foo": "bar"}
        r = authenticated_client.post(upload_url, post_data)

        assert r.status_code == 200

        # make sure error is reported
        messages = list(r.context["messages"])
        assert len(messages) == 1
        assert str(messages[0]).rstrip() == "The video could not be saved due to errors."

    def test_post_upload_video(self, authenticated_client, minimal_mp4):
        upload_url = reverse("castvideo:chooser_upload")
        prefix = "media-chooser-upload"
        post_data = {
            f"{prefix}-title": "foobar",
            f"{prefix}-tags": "foo,bar,baz",
            f"{prefix}-original": minimal_mp4,
        }
        r = authenticated_client.post(upload_url, post_data)

        assert r.status_code == 200

        # make sure field were saved correctly
        video = Video.objects.first()
        assert video.title == post_data[f"{prefix}-title"]

        actual_tags = {t.name for t in video.tags.all()}
        expected_tags = set(post_data[f"{prefix}-tags"].split(","))
        assert actual_tags == expected_tags



# Generated by Django 3.2.6 on 2021-08-21 06:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailredirects", "0006_redirect_increase_max_length"),
        ("wagtailcore", "0062_comment_models_and_pagesubscription"),
        ("wagtailforms", "0004_add_verbose_name_plural"),
        ("cast", "0014_remove_gallery_user"),
    ]

    operations = [
        migrations.DeleteModel(
            name="BlogIndexPage",
        ),
    ]



from typing import Any, Optional

from django import forms
from django.http import HttpResponse
from django.shortcuts import render
from django.views.decorators.http import require_GET
from wagtail.images.models import Image

from ..blocks import get_srcset_images_for_slots
from .htmx_helpers import HtmxHttpRequest


class CommaSeparatedIntegerField(forms.Field):
    def to_python(self, value: Any | None) -> list[int]:  # Any | None from super -> do not override
        if value is None or not value.strip():
            return []
        return [int(item.strip()) for item in value.split(",")]


class GalleryModalForm(forms.Form):
    image_pks = CommaSeparatedIntegerField()
    current_image_pk = forms.IntegerField()
    block_id = forms.CharField()

    def clean(self):
        cleaned_data = super().clean()
        image_pks = cleaned_data.get("image_pks", [])
        current_image_pk = cleaned_data.get("current_image_pk")
        if current_image_pk not in image_pks:
            raise forms.ValidationError(f"current_image_pk {current_image_pk} is not in image_pks {image_pks}")
        return cleaned_data


def get_prev_next_pk(image_pks: list[int], current_image_pk: int) -> tuple[Optional[int], Optional[int]]:
    """
    Given a list of image pks and the current image pk, return the prev and next image pk.
    """
    if len(image_pks) < 2:
        return None, None

    current_index = image_pks.index(current_image_pk)
    prev_pk = image_pks[current_index - 1] if current_index > 0 else None
    next_pk = image_pks[current_index + 1] if current_index < len(image_pks) - 1 else None
    return prev_pk, next_pk


@require_GET
def gallery_modal(request: HtmxHttpRequest, template_base_dir: str) -> HttpResponse:
    """
    This view is used to render the modal for a gallery of images. It is requested
    via htmx and opens the modal with the current image and buttons for the prev/next
    images.

    If the form is not valid, it returns a 400 -> htmx will not swap the content.
    """
    form = GalleryModalForm(request.GET)
    if not form.is_valid():
        return HttpResponse(status=400)
    image_pks = form.cleaned_data["image_pks"]
    current_image_pk = form.cleaned_data["current_image_pk"]
    block_id = form.cleaned_data["block_id"]
    print("gallery_modal for current_image, image_pks and block_id: ", current_image_pk, image_pks)
    prev_pk, next_pk = get_prev_next_pk(image_pks, current_image_pk)

    images_to_fetch = [pk for pk in (prev_pk, current_image_pk, next_pk) if pk is not None]
    images = list(Image.objects.filter(pk__in=images_to_fetch).prefetch_renditions())

    for image in images:
        fetched_renditions = {r.filter_spec: r for r in image.renditions.all() if r.image_id == image.pk}
        images_for_slots = get_srcset_images_for_slots(image, "regular", fetched_renditions=fetched_renditions)
        [image.modal] = images_for_slots.values()

    pk_to_image = {image.pk: image for image in images}
    context = {
        "current_image": pk_to_image[current_image_pk],
        "prev_image": pk_to_image[prev_pk] if prev_pk is not None else None,
        "next_image": pk_to_image[next_pk] if next_pk is not None else None,
        "image_pks": ",".join([str(pk) for pk in image_pks]),
        "template_base_dir": template_base_dir,
        "block_id": block_id,
    }
    return render(request, f"cast/{template_base_dir}/gallery_modal.html", context=context)



from django.conf import settings
from django.contrib import admin
from django.urls import include, path
from django.views.generic import TemplateView
from rest_framework.authtoken import views as authtokenviews
from rest_framework.documentation import include_docs_urls
from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.api.v2.views import PagesAPIViewSet
from wagtail.documents import urls as wagtaildocs_urls

# openapi endpoint broken until this is fixed:
# https://github.com/wagtail/wagtail/issues/8583
PagesAPIViewSet.schema = None


urlpatterns = [
    # path("", RedirectView.as_view(url="/about"), name="home"),
    path(
        "about/",
        TemplateView.as_view(template_name="pages/about.html"),
        name="about",
    ),
    # Django Admin, use {% url 'admin:index' %}
    path(settings.ADMIN_URL, admin.site.urls),
    # Cast
    path("api/api-token-auth/", authtokenviews.obtain_auth_token, name="api-token-auth"),
    path("api-auth/", include("rest_framework.urls", namespace="rest_framework")),
    path("docs/", include_docs_urls(title="My Blog API service")),
    # Cast
    path("cast/", include("cast.urls", namespace="cast")),
    # Threadedcomments
    path("show/comments/", include("fluent_comments.urls")),
    # Wagtail
    path(settings.WAGTAILADMIN_BASE_URL, include(wagtailadmin_urls)),
    path("documents/", include(wagtaildocs_urls)),
    path("", include(wagtail_urls)),  # default is wagtail
]


if settings.DEBUG:
    from django.conf.urls.static import static
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    # Serve static and media files from development server
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from datetime import datetime, time
from typing import Optional, cast

from django import forms
from django.core.exceptions import ValidationError
from django.db.models import QuerySet
from django.forms.models import modelform_factory
from django.utils.translation import gettext_lazy as _
from wagtail.admin import widgets
from wagtail.admin.forms.collections import BaseCollectionMemberForm
from wagtail.admin.forms.search import SearchForm
from wagtail.models import Collection

from .models import Audio, ChapterMark, Video, get_template_base_dir_choices


class VideoForm(forms.ModelForm):
    class Meta:
        model = Video
        fields = ["original"]


class FakePermissionPolicy:
    @staticmethod
    def collections_user_has_permission_for(_user, _action) -> QuerySet[Collection]:
        return Collection.objects.all()


class BaseVideoForm(BaseCollectionMemberForm):
    class Meta:
        widgets = {
            "tags": widgets.AdminTagWidget,
            "original": forms.FileInput,
            "poster": forms.ClearableFileInput,
        }

    permission_policy = FakePermissionPolicy()


def get_video_form() -> type[forms.ModelForm]:
    fields = Video.admin_form_fields
    if "collection" not in fields:
        # force addition of the 'collection' field, because leaving it out can
        # cause dubious results when multiple collections exist (e.g. adding the
        # media to the root collection where the user may not have permission) -
        # and when only one collection exists, it will get hidden anyway.
        fields = cast(tuple[str, str, str, str], tuple(list(fields) + ["collection"]))

    return modelform_factory(
        Video,
        form=BaseVideoForm,
        fields=fields,
    )


class ChapterMarkForm(forms.ModelForm):
    class Meta:
        model = ChapterMark
        fields = ("start", "title", "link", "image")


class FFProbeStartField(forms.TimeField):
    def to_python(self, value) -> time:
        try:
            # utcfromtimestamp, super important!
            return datetime.utcfromtimestamp(float(value)).time()
        except ValueError:
            raise ValidationError(
                _(f"Invalid chaptermark start: {value}"),
                code="invalid",
                params={"start": value},
            )


class FFProbeChapterMarkForm(forms.ModelForm):
    start = FFProbeStartField()

    class Meta:
        model = ChapterMark
        fields = ("start", "title")


def parse_chaptermark_line(line: str) -> ChapterMark:
    def raise_line_validation_error():
        raise ValidationError(
            _(f"Invalid chaptermark line: {line}"),
            code="invalid",
            params={"line": line},
        )

    line_parts = line.split()
    if len(line_parts) < 2:
        raise_line_validation_error()
    start, *parts = line_parts
    title = " ".join(parts)
    form = ChapterMarkForm({"start": start, "title": title})
    if form.is_valid():
        return form.save(commit=False)
    else:
        return raise_line_validation_error()


class ChapterMarksField(forms.CharField):
    def to_python(self, value: Optional[str]) -> list[ChapterMark]:  # type: ignore[override]
        if value is None:
            return []
        chaptermarks = []
        for line in value.split("\n"):
            if len(line) == 0:
                # skip empty lines
                continue
            chaptermarks.append(parse_chaptermark_line(line))
        return chaptermarks


class AudioForm(BaseCollectionMemberForm):
    chaptermarks = ChapterMarksField(widget=forms.Textarea, required=False)
    permission_policy = FakePermissionPolicy()

    class Meta:
        model = Audio
        fields = list(Audio.admin_form_fields) + ["collection"]
        widgets = {
            "tags": widgets.AdminTagWidget,
            "m4a": forms.ClearableFileInput,
            "mp3": forms.ClearableFileInput,
            "oga": forms.ClearableFileInput,
            "opus": forms.ClearableFileInput,
        }

    def get_chaptermarks_from_field_or_files(self, audio: Audio) -> list[ChapterMark]:
        chaptermarks = self.cleaned_data["chaptermarks"]
        if len(chaptermarks) == 0:
            # get chaptermarks from one of the changed files
            changed_audio_formats = []
            for audio_format in audio.audio_formats:
                form_file = self.cleaned_data.get(audio_format)
                if form_file is None:
                    continue
                if form_file != getattr(audio, audio_format):
                    changed_audio_formats.append(audio_format)
            if len(changed_audio_formats) == 0:
                return []
            chaptermark_data = audio.get_chaptermark_data_from_file(changed_audio_formats[0])
            for data in chaptermark_data:
                form = FFProbeChapterMarkForm(data)
                if form.is_valid():
                    chaptermarks.append(form.save(commit=False))
        return chaptermarks

    def save_chaptermarks(self, audio: Audio) -> None:
        # Chapter marks should be saved following this logic:
        #  1. If there are manually added chapter marks in the form, just use them
        #  2. If the form has no chapter marks, but an audio file was changed or
        #     added, look for chapter marks in the audio content
        #  3. Sync changed chapter marks back to database
        chaptermarks = self.get_chaptermarks_from_field_or_files(audio)
        for cm in chaptermarks:
            cm.audio = audio
        ChapterMark.objects.sync_chaptermarks(audio, chaptermarks)

    def save(self, commit=True) -> Audio:
        audio = super().save(commit=commit)
        if commit:
            self.save_chaptermarks(audio)
        return audio


class NonEmptySearchForm(SearchForm):
    """
    A simple search form that takes a single search term.

    It raises an ValidationError if the search term is empty
    or only consists of spaces. Needed after SearchForm changed
    behaviour in Wagtail 4.2.
    """

    def clean_q(self) -> str:
        """
        Make sure the search term is not empty.
        """
        query = self.cleaned_data["q"].strip()
        if not query:
            raise forms.ValidationError(_("Please enter a search term"))
        return query


class SelectThemeForm(forms.Form):
    template_base_dir = forms.ChoiceField(
        choices=[],
        label=_("Theme"),
        help_text=_("Select a theme for this site."),
        required=True,
    )
    next = forms.CharField(widget=forms.HiddenInput, required=False)

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.fields["template_base_dir"].choices = get_template_base_dir_choices()  # type: ignore



from typing import Union

from django.core.paginator import Page, Paginator
from django.db.models import QuerySet
from django.http import HttpRequest

from ..appsettings import MENU_ITEM_PAGINATION
from ..models import Audio, Video

DEFAULT_PAGE_KEY = "p"

pagination_template = "wagtailadmin/shared/ajax_pagination_nav.html"


def paginate(
    request: HttpRequest,
    items: Union[QuerySet[Audio], QuerySet[Video]],
    page_key: str = DEFAULT_PAGE_KEY,
    per_page: int = MENU_ITEM_PAGINATION,
) -> tuple[Paginator, Page]:
    paginator: Paginator = Paginator(items, per_page)
    page = paginator.get_page(request.GET.get(page_key))
    return paginator, page



from django.core.management.base import BaseCommand

from ...models import Video


class Command(BaseCommand):
    help = "recalc the poster images for videos from the videos"

    def handle(self, *args, **options):
        for video in Video.objects.all():
            # orig = video.original
            video.create_poster()
            video.save(poster=False)
            # break



import pytest

from cast.models.moderation import Evaluation, NaiveBayes, SpamFilter, normalize


@pytest.mark.parametrize(
    "train, expected_initial_probabilities",
    [
        ([], {}),
        ([("ham", ""), ("spam", "")], {"ham": 0.5, "spam": 0.5}),
        ([(i, "") for i in range(3)], {k: 1 / 3 for k in range(3)}),
    ],
)
def test_initial_probabilities(train, expected_initial_probabilities):
    model = NaiveBayes().fit(train)
    assert model.predict("") == expected_initial_probabilities


@pytest.mark.parametrize(
    "probabilities, counts_per_label, number_of_words, expected",
    [
        ({}, {}, {}, {}),
        ({"spam": 1}, {"spam": 10}, 100, {"spam": 0.1}),
        (
            {"spam": 1, "ham": 0.5},
            {"spam": 10, "ham": 5},
            100,
            {"spam": 0.1, "ham": 0.025},
        ),
        # no word count -> make sure probability is > 0
        ({"spam": 1}, {}, 100, {"spam": 0.005}),
    ],
)
def test_predict_word_probabilities(probabilities, counts_per_label, number_of_words, expected):
    model = NaiveBayes()
    updated = model.update_probabilities(probabilities, counts_per_label, number_of_words)
    assert updated == expected


@pytest.mark.parametrize(
    "message, expected_probabilities",
    [("foo", {"ham": (1 / 3) * (0.5 / 6), "spam": (2 / 3) * (2 / 6)})],
)
def test_predict(message, expected_probabilities):
    train = [
        ("spam", "foo bar baz"),
        ("spam", "foo asdf bsdf"),
        ("ham", "asdf csdf"),
    ]
    model = NaiveBayes().fit(train)
    expected_probabilities = normalize(expected_probabilities)
    probabilities = model.predict(message)
    for label, probability in probabilities.items():
        assert probability == pytest.approx(expected_probabilities[label])


@pytest.mark.parametrize(
    "train, message, expected_label",
    [
        ([], "asdf", None),
        ([("spam", "foo bar baz")], "asdf", "spam"),
    ],
)
def test_predict_label(train, message, expected_label):
    model = NaiveBayes().fit(train)
    label = model.predict_label(message)
    assert label == expected_label


def test_lots_of_unknown_tokens_lead_to_ham_if_spam_has_more_words():
    """
    There was a bug calculating the probabilities of unknown tokens:

    The probability for unknown tokens was calculated as 0.5 / number_of_words_in_class.
    Since there are much more tokens in spam than in ham, the probability for unknown
    tokens was much higher for ham than for spam. This lead to comments with lots of unknown tokens
    to be classified as ham. Doh.
    """
    train = [
        ("spam", " ".join([str(i) for i in range(100)])),
        ("ham", "asdf bsdf"),
    ]
    model = NaiveBayes().fit(train)
    message = "foo bar baz 87"  # should be spam because of 87
    predicted_label = model.predict_label(message)
    assert predicted_label == "spam"


@pytest.mark.django_db()
def test_model_default_serialization():
    class StubModel:
        foo = "blub"

    spamfilter = SpamFilter(name="stub model", model=StubModel())
    with pytest.raises(TypeError):
        # StubModel is not json serializable -> TypeError
        # makes sure return super().default(obj) for ModelEncoder is called
        spamfilter.save()


@pytest.mark.django_db()
def test_model_naive_bayes_serialization():
    train = [
        ("spam", "foo bar baz"),
        ("ham", "asdf bsdf csdf"),
    ]
    model = NaiveBayes().fit(train)
    spamfilter = SpamFilter(name="naive bayes", model=model)
    spamfilter.save()

    spamfilter.refresh_from_db()
    model_from_db = spamfilter.model
    assert model_from_db == model


@pytest.mark.django_db()
def test_spamfilter_retrain_from_scratch(comment, comment_spam):
    """
    Make sure data from comments is used when retraining
    spamfilter from scratch.
    """
    model = NaiveBayes().fit([])
    spamfilter = SpamFilter(name="naive bayes", model=model)
    spamfilter.save()
    assert spamfilter.model.prior_probabilities == {}

    train = SpamFilter.get_training_data_comments()
    spamfilter.retrain_from_scratch(train)
    assert spamfilter.model.prior_probabilities == {"ham": 0.5, "spam": 0.5}


def test_evaluate_model_non_empty_test_messages():
    class Model:
        prior_probabilities = {"spam": 0.5, "ham": 0.5}

        @staticmethod
        def predict_label(message):
            return "spam"

    messages = [("spam", "foo bar baz"), ("ham", "asdf bsdf csdf")]
    label_results = Evaluation.evaluate_model(Model(), messages)
    assert label_results["spam"]["true_positive"] == 1  # first message is tp
    assert label_results["spam"]["false_positive"] == 1  # second message is fp


def test_get_precision_recall_f1():
    counts = {
        "ham": {"true_positive": 1, "true_negative": 1, "false_positive": 1, "false_negative": 1},
    }
    precision, recall, f1 = Evaluation.get_precision_recall_f1(counts["ham"])
    assert (precision, recall, f1) == (0.5, 0.5, 0.5)


def test_calc_performance():
    counts = {
        "spam": {"true_positive": 1, "true_negative": 1, "false_positive": 1, "false_negative": 1},
        "ham": {"true_positive": 1, "true_negative": 1, "false_positive": 1, "false_negative": 1},
    }
    performance = Evaluation().calc_performance(counts)
    assert tuple(performance["spam"].values()) == (0.5, 0.5, 0.5)
    assert tuple(performance["ham"].values()) == (0.5, 0.5, 0.5)


def test_evaluate_with_some_messages():
    class Model:
        prior_probabilities = {"spam": 0.5, "ham": 0.5}

        @staticmethod
        def predict_label(message):
            return "spam"

        def fit(self, _train):
            return self

    messages = [("spam", "foo bar baz"), ("ham", "asdf bsdf csdf")]
    evaluation = Evaluation(model_class=Model, num_folds=1)
    performance = evaluation.evaluate(messages)
    assert performance["spam"]["precision"] == 0.5

    messages = []
    with pytest.raises(ValueError):
        evaluation.evaluate(messages)



# Generated by Django 3.2.4 on 2021-06-28 16:28

import django.db.models.deletion
import wagtail.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0062_comment_models_and_pagesubscription"),
        ("cast", "0005_auto_20201024_0613"),
    ]

    operations = [
        migrations.CreateModel(
            name="BlogIndexPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                ("intro", wagtail.fields.RichTextField(blank=True)),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.AlterField(
            model_name="audio",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="chaptermark",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="file",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="gallery",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="homepage",
            name="alias_for_page",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Make this page an alias for another page, redirecting to it with a non permanent redirect.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="aliases_homepage",
                to="wagtailcore.page",
                verbose_name="Redirect to another page",
            ),
        ),
        migrations.AlterField(
            model_name="image",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="itunesartwork",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="request",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="video",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
    ]



# Generated by Django 4.1.4 on 2023-01-19 05:50

import cast.blocks
from django.db import migrations
import wagtail.blocks
import wagtail.embeds.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0025_add_performance_indicators"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Request",
        ),
        migrations.AlterField(
            model_name="post",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    (
                        "overview",
                        wagtail.blocks.StreamBlock(
                            [
                                ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                (
                                    "code",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "language",
                                                wagtail.blocks.CharBlock(help_text="The language of the code block"),
                                            ),
                                            (
                                                "source",
                                                wagtail.blocks.TextBlock(
                                                    help_text="The source code of the block", rows=8
                                                ),
                                            ),
                                        ],
                                        icon="code",
                                    ),
                                ),
                                ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                                ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(icon="media", template="cast/video/video.html"),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(icon="media", template="cast/audio/audio.html"),
                                ),
                            ]
                        ),
                    ),
                    (
                        "detail",
                        wagtail.blocks.StreamBlock(
                            [
                                ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                                ("paragraph", wagtail.blocks.RichTextBlock()),
                                (
                                    "code",
                                    wagtail.blocks.StructBlock(
                                        [
                                            (
                                                "language",
                                                wagtail.blocks.CharBlock(help_text="The language of the code block"),
                                            ),
                                            (
                                                "source",
                                                wagtail.blocks.TextBlock(
                                                    help_text="The source code of the block", rows=8
                                                ),
                                            ),
                                        ],
                                        icon="code",
                                    ),
                                ),
                                ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                                ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                                ("embed", wagtail.embeds.blocks.EmbedBlock()),
                                (
                                    "video",
                                    cast.blocks.VideoChooserBlock(icon="media", template="cast/video/video.html"),
                                ),
                                (
                                    "audio",
                                    cast.blocks.AudioChooserBlock(icon="media", template="cast/audio/audio.html"),
                                ),
                            ]
                        ),
                    ),
                ],
                use_json_field=True,
            ),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-24 05:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0002_remove_blog_description"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="post",
            name="parent_blog",
        ),
    ]



# Generated by Django 4.1.4 on 2023-02-05 09:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0036_alter_blog_author"),
    ]

    operations = [
        migrations.AlterField(
            model_name="episode",
            name="block",
            field=models.BooleanField(
                default=False,
                help_text="Check to block this episode from iTunes because <br />its content might cause the entire show to be <br />removed from iTunes.",
            ),
        ),
        migrations.AlterField(
            model_name="episode",
            name="explicit",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "yes"), (2, "no"), (3, "clean")],
                default=1,
                help_text="``Clean`` will put the clean iTunes graphic by it.",
            ),
        ),
        migrations.AlterField(
            model_name="episode",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="podcast",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-delimited list of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
                verbose_name="keywords",
            ),
        ),
        migrations.AlterField(
            model_name="post",
            name="comments_enabled",
            field=models.BooleanField(default=True, help_text="Whether comments are enabled for this post."),
        ),
    ]



import pytest


class TestTwitterPlayerCard:
    pytestmark = pytest.mark.django_db

    def test_includes_card_with_podcast_audio(self, client, episode):
        episode = episode
        detail_url = episode.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "html" in content
        assert episode.title in content
        assert "Twitter Player Card" in content



from django.urls import include, path
from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls

urlpatterns = [
    path("cast/comments/", include("fluent_comments.urls")),
    path("cast/", include("cast.urls", namespace="cast")),
    path("cms/", include(wagtailadmin_urls)),
    path("", include(wagtail_urls)),
]



# Generated by Django 3.2.8 on 2021-10-05 09:29

import cast.models.moderation
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0021_spamfilter"),
    ]

    operations = [
        migrations.AlterField(
            model_name="spamfilter",
            name="model",
            field=models.JSONField(
                default=dict, encoder=cast.models.moderation.ModelEncoder, verbose_name="Spamfilter Model"
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-06 14:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0009_alter_post_body"),
    ]

    operations = [
        migrations.RenameField(
            model_name="blog",
            old_name="intro",
            new_name="description",
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-01 08:34

import wagtail.blocks
import wagtail.fields
import wagtail.embeds.blocks
import wagtail.images.blocks
from django.db import migrations

import cast.blocks


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0006_auto_20210628_1628"),
    ]

    operations = [
        migrations.AlterField(
            model_name="post",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    ("heading", wagtail.blocks.CharBlock(form_classname="full title")),
                    ("paragraph", wagtail.blocks.RichTextBlock()),
                    ("image", wagtail.images.blocks.ImageChooserBlock(template="cast/image/image.html")),
                    ("gallery", cast.blocks.GalleryBlock(wagtail.images.blocks.ImageChooserBlock())),
                    ("embed", wagtail.embeds.blocks.EmbedBlock()),
                ]
            ),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-21 05:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0026_delete_request_alter_post_body"),
    ]

    operations = [
        migrations.CreateModel(
            name="Episode",
            fields=[
                (
                    "post_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="cast.post",
                    ),
                ),
                (
                    "new_keywords",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                        max_length=255,
                        verbose_name="keywords",
                    ),
                ),
                (
                    "new_explicit",
                    models.PositiveSmallIntegerField(
                        choices=[(1, "yes"), (2, "no"), (3, "clean")],
                        default=1,
                        help_text="``Clean`` will put the clean iTunes graphic by it.",
                        verbose_name="explicit",
                    ),
                ),
                (
                    "new_block",
                    models.BooleanField(
                        default=False,
                        help_text="Check to block this episode from iTunes because <br />its content might cause the entire show to be <br />removed from iTunes.",
                        verbose_name="block",
                    ),
                ),
                (
                    "new_podcast_audio",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="episodes",
                        to="cast.audio",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cast.post",),
        ),
    ]



import pytest


class TestPostDetail:
    pytestmark = pytest.mark.django_db

    def test_get_post_detail(self, client, post):
        detail_url = post.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "html" in content
        assert post.title in content

    def test_get_post_detail_with_detail(self, client, post):
        detail_url = post.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "in_all" in content
        assert "only_in_detail" in content

    def test_post_detail_with_gallery(self, client, post_with_gallery):
        detail_url = post_with_gallery.get_url()

        r = client.get(detail_url)
        assert r.status_code == 200

        content = r.content.decode("utf-8")
        assert "cast-gallery-thumbnail" in content



# Generated by Django 4.1.7 on 2023-03-12 21:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0041_templatebasedirectory"),
    ]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="template_base_dir",
            field=models.CharField(
                choices=[("bootstrap4", "Bootstrap 4"), ("plain", "Just HTML")], default="bootstrap4", max_length=10
            ),
        ),
    ]



from django.http import Http404, HttpRequest, HttpResponse
from django.shortcuts import get_object_or_404, render
from django.views.decorators.http import require_GET

from ..models import Blog, Episode


@require_GET
def twitter_player(request: HttpRequest, blog_slug: str, episode_slug: str) -> HttpResponse:
    """
    This view is used to render the twitter card player. This is a
    podlove player consisting of just the play button. But it needs
    the episode data from the server.
    """
    blog = get_object_or_404(Blog, slug=blog_slug)
    episode = get_object_or_404(Episode, slug=episode_slug)
    if episode.blog != blog:
        raise Http404("Episode not found")
    context = {"episode": episode}
    return render(request, "cast/twitter/card_player.html", context=context)



# Generated by Django 4.1.5 on 2023-01-04 13:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0024_alter_homepage_body_alter_post_body"),
    ]

    operations = [
        migrations.AddField(
            model_name="spamfilter",
            name="performance",
            field=models.JSONField(default=dict, verbose_name="Spamfilter Performance Indicators"),
        ),
    ]



import pytest
from django.urls import reverse


@pytest.mark.django_db
def test_twitter_player(client, episode):
    episode = episode
    url = reverse("cast:twitter-player", kwargs={"episode_slug": episode.slug, "blog_slug": episode.blog.slug})
    r = client.get(url)
    assert r.status_code == 200

    content = r.content.decode("utf-8")
    assert str(episode.uuid) in content
    assert "embed.5.js" in content
